c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine avergf(m,nxf,nyf,nzf,nspecf,presf,temprf, 
     &                  cncf,coldf,avconf,avgtmf,deltf,time,ndate)
c
c-----ATDM v1.00
c
c     Computes the average fine grid concentrations using
c     the rectangle rule for the time integrations.
c
c-----95/02/22:
c     -from UAM-V Revision 1.3  94/07/27  09:12:23  tcm
c     -removed code that keeps track of running hourly O3 averages
c-----95/04/05:
c     -altered running average calculation for ug/m3, rather than ppm
c
c     when  m=1 we begin the time average
c           m=2 we continue to generate the average
c           m=3 we conclude the averaging
c
c     Input arguments:
c               m:              averaging flag   
c               nxf:            number of fine grid cell in x-direction
c               nyf:            number of fine grid cell in y-direction
c               nzf:            number of fine grid layers
c               nspecf:         number of modeled species
c               presf:          array of fine grid pressure (mb)
c               temprf:         array of fine grid temperature (K)
c               cncf:           array of fine grid concentrations (ppm)
c               coldf:          array of old concentrations (ppm)
c               deltf:          fine grid time step (min)
c               time:           model time
c               ndate:          model date
c     Output arguments:
c               avconf:         fine grid average concentrations (ug/m3)
c               avgtmf:         total averaging time (min)
c     Files written:
c               6:              standard output file unit (*)
c     Called by:
c               atdm
c
      save
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
c
      dimension cncf(nxf,nyf,nspecf,nzf), coldf(nxf,nyf,nspecf),
     &          avconf(nxf,nyf,mxspec), presf(nxf,nyf,nzf),
     &          temprf(nxf,nyf,nzf)
      real mfac
c
      write(*,4000) time,ndate,deltf,m
c
      if (m.eq.1) then
        avgtmf=0.0
        do 10 k=1,nspecf
          do 9 j=1,nyf
            do 8 i=1,nxf
              avconf(i,j,k) = 0.0
    8       continue
    9     continue
   10   continue
      endif
c
c     add in cold arrays to avcon arrays
c
      if (m .gt. 1) then
        avgtmf=avgtmf+deltf
        do 112 k=1,nspecf
          do 111 j=2,nyf-1
            do 110 i=2,nxf-1
              avconf(i,j,k) = avconf(i,j,k) + 0.5*deltf*coldf(i,j,k)
  110       continue
  111     continue
  112   continue
c
c     add in current instantenous concentrations
c
        do 290 k=1,nspecf
          ksp = jind(k)
          do 280 j=2,nyf-1
            do 270 i=2,nxf-1
              mfac = cpfac*(1013.25/presf(i,j,1))*(temprf(i,j,1)/273.)
              avconf(i,j,k) = avconf(i,j,k) + 0.5*deltf*molwt(ksp)*
     &                        cncf(i,j,k,1)/mfac
  270       continue
  280     continue
  290   continue
      endif
c
c     close out averaging
c
      if (m.eq.3) then
        do 312 k=1,nspecf
          do 311 j=2,nyf-1
            do 310 i=2,nxf-1
              avconf(i,j,k) = avconf(i,j,k)/avgtmf
  310       continue
  311     continue
  312   continue
      endif
c
c     load cncf into cold arrays for next time step
c
      do 350 k=1,nspecf
        ksp = jind(k)
        do 345 j=2,nyf-1
          do 340 i=2,nxf-1
            mfac = cpfac*(1013.25/presf(i,j,1))*(temprf(i,j,1)/273.)
            coldf(i,j,k) = molwt(ksp)*cncf(i,j,k,1)/mfac
  340     continue
  345   continue
  350 continue
c
 4000 format(' enter avergf ',f10.1,i10,f10.2,'  m = ',i3)
 4100 format(' leave avergf')
      avetime = time
      return
      end
