c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine chread(nuchp,nspec,nnhg,idebug)
c
c-----ATDM v1.00
c
c     Reads chemistry parameters and fills in species indices
c
c-----95/03/03:
c     -from UAM-V Revision 1.2  94/02/10  20:53:58  tcm
c     -Major modifications specific to ATDM chemistry
c     -New variables:
c        iszdist(spec)
c        nszdist
c        ngas
c        mtemp
c-----95/08/24:
c     -improved output diagnostic formats
c-----95/09/28:
c     -changes some species names to agree with emissions
c-----96/09/11:dfg
c     -added lmech flag. When lmech is true, chemparm file is different
c      from the parameterized chemparm file; do not need to read from
c      OH lookup tables and ozone concentration files.
c-----97/11/26:dfg
c     -fixed a bug which occured with toxic chemistry on (ltxchm=true).
c-----97/12/20:dfg
c     -disable line 232-234 in responding to unit of per min for k's.
c-----99/12/15:tcm
c     -make Hg pointers arrays to allow tagging of region or category
c
c     Input arguments:
c               nuchp:          chemparam file unit number    
c               idebug:         diagnostic debug flag
c     Output arguments:
c               nspec:          number of species read from chemparam file
c     Files read:
c               nuchp:          chemparam file
c     Files written:
c               6:              standard output file unit (*)
c               8:              diagnostic file
c     Called by:
c               setup
c
      include 'parchm.cmd'
      include 'com3.cmd'
      include 'comchem.cmd'
      include 'ctlflg.cmd'
      include 'aerosl.cmd'
      include 'parfin.cmd'
      include 'kweight.cmd'
c
      dimension nam(10),namchp(10),nid(60)
      character*10 mtemp
      data namchp/1hC,1hH,1hE,1hM,1hP,1hA,1hR,1hA,1hM,1h /
c
c-----Read and check file description header records
c     bit 12 (2048's) of idebug activates writes to debug file in chread
c
      idiag = and(idebug,2048)
      write (*,*) 'entered chread'
      if (idiag.ne.0) then
        write(8,*)'just entered chread...'
        write(8,*)'nspint= ',nspint
      endif
c
      ngas = 0
      npart = 0
      write (8,2500)
      read  (nuchp,2505) nam, nid, nspec, nszdist
c      write  (*,*) 'nam, nid, nspec, nszdist', nam, nid, nspec, nszdist
      write (8,2510) nam, nid, nspec, nszdist
c      write (*,*) 'nszdist', nszdist,mxsdist
c***  
c----Read and print numbers of species and reaction when micro
c    cb4 mechanism used (9/6/96, dfg).
c***
      if (lmech) then
        read  (nuchp,2506) nspec1, nreact
        write  (*,*) 'nspec1, nreact', nspec1, nreact
        write (8,2511) nspec1,nreact
      endif
c
      do 12 i=1,10
        if (nam(i).ne.namchp(i)) then
          write (8,2012) nam, namchp
          stop
        endif
   12 continue
      if (nspec.gt.mxspec) then
        write (8,2050) mxspec
        stop
      endif
      if (nszdist.gt.mxsdist) then
         write(*,*)'Number of size distributions > ',mxsdist
         write(*,*)'Reduce NSZDIST, or increase MXSDIST and recompile'
         stop
      endif
      if (idiag.ne.0) write(8,*)'checked file description header'
c
c-----Read default aromatic fraction for SOA calculation
c
      read (nuchp,*)
      read (nuchp,'(f10.0)') afdef
c
c-----Read particulate size bins and mass distributions
c
      read (nuchp,*)
      read (nuchp,774) (rsize(iptcl),iptcl=1,nbin)
      read (nuchp,*)
      write(8,775) (rsize(iptcl),iptcl=1,nbin)
      write(8,776)
      do 772 n = 1,nszdist
         read (nuchp,774) (rmass(iptcl,n),iptcl=1,nbin)
         write(8,778) n, (rmass(iptcl,n),iptcl=1,nbin)
         flrge(n) = 0.
         do 773 iptcl = 1,nbin
            if (rsize(iptcl).gt.1.)
     &         flrge(n) = flrge(n) + rmass(iptcl,n) 
  773    continue
         write(8,777) flrge(n)
  772 continue
c
c-----Read and print chemparam species definition
c
      read (nuchp,*)
      if (lmech) then
        write(8,2519)
      else
        write (8,2520)
      endif
c
      write (*,*) 'NSPEC =', nspec
      do 15 l = 1,nspec
c***
c-----Read chemical species and its definition for micro CB4 mechanism.
c     (9/6/96, dfg)
c
        if (lmech) then
          if (l.le.nspec1) then
          read (nuchp,2535) (mspec(i,l),i=1,10),lreac(l),lssic(l),
     &                      lssbc(l), bdnl(l)
          write (8,2535)    (mspec(i,l),i=1,10),lreac(l),lssic(l),
     &                      lssbc(l), bdnl(l)
          else
            read (nuchp,2515) (mspec(i,l), i=1,10),iszdin,rhopin,
     &                        fc(l),bdnl(l)
            write (8,2521)    (mspec(i,l), i=1,10),iszdin,rhopin,
     &                        fc(l),bdnl(l)
          endif
c***
        else
          read (nuchp,2515) (mspec(i,l), i=1,10),iszdin,rhopin,
     &                      fc(l),bdnl(l)
          write (8,2521)    (mspec(i,l), i=1,10),iszdin,rhopin,
     &                      fc(l),bdnl(l)
        endif
c
c-----Match specified names with internal names
c
        write (*,*) 'within chread, mxgas, nspint=', mxgas, nspint
c
         do 140 lsp = 1,nspint
c            write (*,*) 'lsp=', lsp, nspint
            do 145 i = 1,10
              if (mspec(i,l) .ne. mspint(i,lsp)) then
c                 write (*,*) lsp, i, l, mspec(i,l), mspint(i,lsp)
                 go to 140
              end if
  145       continue
            if (kind(lsp) .ne. 0) then
               l2 = kind(lsp)
               write(8,2250) l, (mspec(i,l),i=1,10),
     &                           l2, (mspec(i,l2),i=1,10)
               stop
            endif
            kind(lsp) = l
            jind(l) = lsp
            write(mtemp,'(10a1)') (mspec(i,l),i=1,10)
c***
c           if (lmech) mxxgas=mxgas
c           if (.not.lmech) mxxgas=mxgas1
c***
            write (*,*) 'mxgas=', mxgas
            if (lsp.le.mxgas) then
               if (npart.ne.0) then
                  write(*,*)'ERROR: Species in chemparam file must',
     &                      ' be ordered such that all gaseous species'
                  write(*,*)'appear first, followed by all particulate',
     &                      ' species;'
                  write(*,'(a,i3,a,a4,a)')
     &                    'CHREAD diagnoses that gas species # ',
     &                      l,' labelled ',MTEMP,' appears after'
                  write(*,*)'a particulate species'
                  stop
               endif
               ngas = ngas + 1
            else
               npart = npart + 1
               iszdist(l) = iszdin
               rhop(l) = rhopin
            endif
            write(*,'(a,i3,a,a10,a,i3)')
     &           'Input species # ',l,' labelled ',mtemp,
     &           ' mapped to internal species # ',lsp
c
c-----Check particle size distribution indices for consistency with control 
c     inputs
c
            if (ldepn .and. lsp.gt.mxgas .and. mtemp.ne.'NH4N'
     &          .and. mtemp.ne.'NH4S') then
c               write (*,*) ldepn
c               write (*,*) lsp, mxgas
c               write (*,*) mtemp
               if (iszdin.eq.0) then
                  write(8,*)'Size distribution indices must be',
     &                      ' specified for species number',l
                  write(8,'(a,i2)')'Supply indices from 1 to ',
     &                                 nszdist
                  stop
               elseif (iszdin.gt.nszdist) then
                  write (*,*) 'iszdin nszdist', iszdin, nszdist
                  write(8,'(a,i3)')
     &                 'Size distribution index for species ',l
                  write(8,'(a,i3)')'is greater than specified number',
     &                      ' of distributions: ',nszdist 
                  write(8,'(a,i2)')'Supply indices from 1 to ',
     &                                 nszdist
                  stop
               endif
            endif

            goto 15
  140    continue
         write(8,2220) l, (mspec(i,l),i=1,10)
         stop
  15  continue
      write (*,*) 'ngas =', ngas, 'npart=', npart
      if (idiag.ne.0) 
     &   write(8,*)'finished reading chem definition records'
c***
c----Read reaction record for micro CB4 mechanism.
c    (9/6/96, dfg)
      if (lmech) then
        read (nuchp,*)
        write(8,2540)
        do 90 j=1,nreact
          read (nuchp,2545) rkn(j), lphot(j), ltdep(j), lpdep(j),
     &                   norder(j),acten(j)
          write (8,2550) j, rkn(j), lphot(j), ltdep(j), lpdep(j),
     &                   norder(j),acten(j)
 90     continue
      else
        go to 120
      endif
c 
c   Convert rate constants from unit of /ppm-min to unit of /ppm-hr
c   Because of step4 use time step in hour
c
c***DFG     do j = 1, nreact
c***DFG       rkn(j)=rkn(j)*60.
c***DFG     end do
cmake corrections
c
c      do 100 j=1,nreact
c
c  For now we do not calculate pressure dependency for rate constants.
c
c        if (lpdep(j)) then
c          lpdep(j) = .false.
c        endif
c
c  For now we do not consider the higher order reactions.
c
c        if (norder(j).gt.1) then
c          norder(j) = 1
c        endif
c
c  100 continue
c
c     *** set global reaction flags and initialize calculated rk to rkn
c
c      lanyph = .false.
c      lanytd = .false.
c
c      do 110 j=1,nreact
c        rk(j) = rkn(j)
c        if (rkn(j).eq.0.) go to 110
c        if (lphot(j)) lanyph = .true.
c         if (acten(j).eq.0.) ltdep(j) = .false.
c        if (ltdep(j)) lanytd = .true.
c  110 continue
cmake corrections
c
      write (*,*) 'after allrate'
c   Set Micro CB4 reaction Flags
c
       ncall=1
       lso2=.true.
       lsulf=.true.
       lntr=.true.
c
c   Validate chemistry integration controls
c
       write (*,*) dtmin4, maxitr, rerror
      ibad=0
      if (dtmin4.lt.0.) ibad = ibad + 1
      if (maxitr.lt.1)  ibad = ibad + 1
      if (rerror.le.0.) ibad = ibad + 1
      if (ibad.gt.0) then
        write(*,*) ibad
        stop
      endif
c
  120 continue
      write (*,*) '120'
c
c-----After identifying all input species, do some reality checks
c
c     1) check that number of gas+particulate species match input of 
c        total species in chemparam file
c
      if (ngas+npart.ne.nspec) then
         write(*,*)'ERROR: sum of specified gas and particulate',
     &             ' species is not equal to'
         write(*,*)' specified total number of species: ',
     &              ngas,npart,nspec
         stop
      endif
      write (*,*) 'before 2'
c
c     2) check that all required species are present if PM chemistry
c        has been specified
c***
      if (lpmchm) then
        if (.not.lmech) then
          if (kno.eq.0 .or.
     &        kno2.eq.0 .or.
     &        kso2.eq.0 .or.
     &        knh3.eq.0 .or.
     &        kvoc.eq.0 .or.
     &        khno3.eq.0) then
             write(*,400)
             stop
          endif
        endif
        if (kpno3.eq.0 .or.
     &      kgso4.eq.0 .or.
     &      kaso4.eq.0 .or.
     &      knh4n.eq.0 .or.
     &      knh4s.eq.0) then
           write(*,400)
           stop
        endif
      endif
c***
      write (*,*) 'before 3'
      if (ltxchm) then
c
c     3) check that all required PM-related species are present if
c        semi-volatile toxics have been specified 
c
         if (kpom1.ne.0 .or.
     &       kpom2.ne.0 .or.
     &       kdiox.ne.0 .or.
     &       katraz.ne.0) then
            if ((kpom1.ne.0 .and. kpom1p.eq.0) .or.
     &          (kpom2.ne.0 .and. kpom2p.eq.0) .or.
     &          (kdiox.ne.0 .and. kdioxp.eq.0) .or.
     &          (katraz.ne.0 .and. katrazp.eq.0)) then
               write(*,*)'ERROR: You have invoked TOX chemistry ' 
               write(*,*)'and have specified semi-volatile toxics --'
               write(*,*) 'POM1 requires POM1P'
               write(*,*) 'POM2 requires POM2P'
               write(*,*) 'DIOXIN requires DIOXP'
               write(*,*) 'ATRAZ requires ATRAZP'
               stop
            endif
c***
            if (.not.lmech) then
              if (.not.  lo3) then
                if (kno.eq.0 .or.
     &              kno2.eq.0 .or.
     &              kso2.eq.0 .or.
     &              kvoc.eq.0) then
                  write(*,500)
                  stop
                endif
              endif
            endif
            if (.not.  lo3) then
              if (kgso4.eq.0 .or.
     &            kaso4.eq.0 .or.
     &            kpfin.eq.0 .or.
     &            kpcrs.eq.0) then
                write(*,500)
                stop
              endif
            endif
c***
            if (.not.lmech) then
              if (khno3.ne.0 ) then
                if (khno3.eq.0 .or.
     &            kpno3.eq.0 .or.
     &            knh3.eq.0 .or.
     &            knh4n.eq.0 .or.
     &            knh4s.eq.0) then
                  write(*,600) 
                  stop
                endif
              endif
            endif
            if (kpno3.ne.0 .or.
     &        knh3.ne.0 .or.
     &        knh4n.ne.0 .or.
     &        knh4s.ne.0) then
              if (kpno3.eq.0 .or.
     &            knh3.eq.0 .or.
     &            knh4n.eq.0 .or.
     &            knh4s.eq.0) then
                  write(*,600)
                  stop
              endif
            endif
c
         endif
      write (*,*) 'before 4'
c
c     4) check that all required species are present if Hg chemistry 
c        has been specified
c
         nnhg = 0
         do n=1,mxhg
           if (khg0(n).ne.0 .or. khg2(n).ne.0 .or. khgp(n).ne.0) then
             nnhg = nnhg + 1
             if (khg0(n).eq.0 .or. khg2(n).eq.0 .or. khgp(n).eq.0) then
                write(*,*)'ERROR: You have invoked Hg chemistry -- ',
     &                   'the following species are required:'
                write(*,*) 'HG0       HG2G      HG2P'
                stop
             endif
           endif
         enddo
      write (*,*) 'before 4.1, nnhg = ', nnhg
      write (*,*) 'khg0: ', (khg0(i),i=1,mxhg)
      write (*,*) 'khg2: ', (khg2(i),i=1,mxhg)
      write (*,*) 'khgp: ', (khgp(i),i=1,mxhg)
c
c     5) check that all required species are present if POM chemistry
c        has been specified 
c
         if (knaph.ne.0 .or.
     &       kpom1.ne.0 .or.
     &       kpom2.ne.0 .or.
     &       kpom1p.ne.0 .or.
     &       kpom2p.ne.0) then
            if (knaph.eq.0 .or.
     &          kpom1.eq.0 .or.
     &          kpom2.eq.0 .or.
     &          kpom1p.eq.0 .or.
     &          kpom2p.eq.0) then
               write(*,*)'ERROR: You have invoked POM chemistry -- ',
     &                   'the following species are required:'
              write(*,*) 'NAPH      POM1      POM2      POM1P     POM2P'
               stop
            endif
         endif
c
c     5) check that all required species are present if DIOXIN
c        has been specified 
c
         if (kdiox.ne.0 .or. kdioxp.ne.0) then
            if (kdiox.eq.0 .or. kdioxp.eq.0) then
               write(*,*)'ERROR: You have specified DIOXIN -- ',
     &                   'the following species are required:'
               write(*,*) 'DIOX      DIOXP'
               stop
            endif
         endif
c
      write (*,*) 'before 6'
c     6) check that all required species are present if ATRAZ
c        has been specified 
c
         if (katraz.ne.0 .or. katrazp.ne.0) then
            if (katraz.eq.0 .or. katraz.eq.0) then
               write(*,*)'ERROR: You have specified ATRAZ -- ',
     &                   'the following species are required:'
               write(*,*) 'ATRAZ     ATRAZP'
               stop
            endif
         endif
      endif
c
c     7) determine number of tagged cadmium species
c
      nncd = 0
      do n=1,mxcd
        if (kcd(n).ne.0) then
          nncd = nncd + 1
        endif
      enddo
      write (*,*) 'After 7, nncd = ', nncd
      write (*,*) 'kcd: ', (kcd(i),i=1,mxcd)
c
c------------------------------------------------------------------------
c
c     ***  formats
c
 400   format('ERROR: You have invoked PM chemistry -- ',
     & 'the following species are required:',/,
     & 'NO        NO2       SO2       NH3',/,
     & 'VOC       HNO3      PNO3      GSO4',/,
     & 'ASO4      NH4N      NH4S')
 500  format('ERROR: You have invoked TOX chemistry ',/,
     &  'and have specified semi-volatile toxics --',
     &  ' the following PM species are required:',/,
     &  'NO        NO2       SO2       VOC',/,
     &  'GSO4      ASO4      PMFINE    PMCOARS')
  600 format('ERROR: You have invoked TOX chemistry ',/,
     &  'and have specified semi-volatile toxics',
     &  'and either nitrate or ammonia species --',/,
     &  ' the following species are required:',/,
     &  'NO        NO2       SO2       VOC',/,
     &  'GSO4      ASO4      PFIN      PCRS',/,
     &  'HNO3      PNO3      NH3       NH4N      NH4S')
c
  774 format(8f7.3)
  775 format(' Particulate size bins:',/,6x,8f7.3)
  776 format(' Mass fraction distributions')
  778 format(' # ',i2,':',8f7.3)
  777 format(' Mass fraction > 1 um:',f7.3)
 2012 format ('***ERROR in -chread- *** file named ',
     &        10a1,' should have been ',10a1)
 2050 format ('Maximum dimensions exceeded in CHEMPARAM file',/,
     &        ' max number of species = ',i2)
 2220 format (' ***ERROR*** ',/,' input species number ',i4,
     &        '( ',10a1,') does not match ',/,
     &        ' any internal name')
 2250 format (' ***ERROR*** ',/,' input species number ',i4,
     &        '( ',10a1,') already defined ',/,
     &        ' at number ',i4,' (',10a1,')')
 2500 format (/,'****************  CHEMPARAM FILE  *******************')
 2505 format (10a1,/,60a1,/,10x,i2,/,10x,i2)
 2506 format (10x,i2,/,10x,i3)
 2510 format ('File description header record',/,
     &        5x,'File type        = ', 10a1,/,
     &        5x,'File id          = ', 60a1,/,
     &        5x,'No of species    = ', i2,/,
     &        5x,'No of size dists = ', i2)
 2511 format ('Micro CB4 mechanism: ',/,
     &        5x,'No of species    = ', i2,/,
     &        5x,'No of reactions  = ', i3)
 2515 format (5x,10a1,i5,2f10.0,e10.2)
 2519 format (' Species parameters record',/,
     &  5x,'Spec Name ', 2x,'React', 2x,' SSIC', 2x,' SSBC',
     & 2x, 10x,'Low Bd Num')
 2520 format ('Species descriptions',/,
     &        '     SPECIES     SZDIST   RHOP        FC      BDNL')
 2521 format (5x,10a1,i5,3(1pe10.3))
 2525 format (/23x,60a1,2(/28x,l4),1(/37x,i3))
 2535 format (5x,10a1,3l7,1pe17.4)
 2540 format ('  reaction parameters record',/,
     & 7x, 'Rate const', 2x, 'Phot', 2x, 'Temp',2x, 'Pres', 2x,
     &     ' #', 5x, 'Act Energy')
 2545 format (5x,e12.4,3l6,i6,f12.1)
 2550 format (i4,1x,1pe12.3,3l6,i6,f12.1)
c
      write (*,*) 'end of chread and ngas is =', ngas
c
      return
      end
