c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine crsfil(concf,dpositf,wpositf,sigfin,nxf,nyf,nsp,nzf,
     &                  pstrf,ifine,idebug)
c
c-----ATDM v1.00
c
c     Fills in coarse grid concentrations with fine grid
c     concentrations.  Should be done at end of time step
c     after all transport and chem calculations are done.
c
c-----95/06/20:
c     -from UAM-V Revision 1.2  94/02/10  20:54:14  tcm
c     -revised to fill coarse grid with ppm rather than umol/m3
c     -added presf and temprf to argument list
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c      removed temprf, htf, presf, from argument list, added sigmaf
c      and pstarf
c      added data statements for grav and amolwt
c-----95/11/08:
c     -added fill of dposit and wposit with respective fine grid amounts
c-----95/11/28:
c     -fixed some coding errors related to entry above
c
c     Input arguments:
c               concf:          fine grid concentrations (ppm)      
c               dpositf:        fine grid dry dep flux (g/km2)
c               wpositf:        fine grid wet dep flux (g/km2)
c               sigfin:         fine grid sigma structure
c               nxf:            number of fine grid cells in x-direction
c               nyf:            number of fine grid cells in y-direction
c               nsp:            number of modeled species
c               nzf:            number of fine grid layers
c               pstrf:          fine grid P* (mb)
c               ifine:          fine grid index
c               idebug:         diagnostic debug flag
c     Files written:
c               6:              standard output file unit (*)
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'deffin.cmd'
      include 'ctlflg.cmd'
      dimension concf(nxf,nyf,nsp,nzf)
      dimension dpositf(nxf,nyf,nsp),wpositf(nxf,nyf,nsp)
      dimension sigfin(nzf+1)
      dimension pstrf(nxf,nyf)
      real mfac
      data grav /9.8/, amolwt /28.8/
c
c  Bit ten of idebug on (i.e., the five hundred twelves location) 
c  will turn on debug writes in rchem.
c
      idiag = and(idebug,512)
c
c  zero out coarse grid where overlapped by this fine grid
c
      do 100 k=1,nlayer
        do 90 n=1,nospec
          do 80 j=jyfb(ifine),jyfe(ifine)
            do 70 i=ixfb(ifine),ixfe(ifine)
              conc(i,j,n,k) = 0.
              if (ldepn .and. k.eq.1) dposit(i,j,n) = 0.
              if (lwet .and. k.eq.1) wposit(i,j,n) = 0.
   70       continue
   80     continue
   90   continue
  100 continue 
c
c  sum coarse grid for all levels and all species
c
      do 200 k=1,nlayer
        kf2 = kvmap(k,ifine)
        if (k .eq. 1) then
          kf1 = 1
        else 
          kf1 = kvmap(k-1,ifine) + 1
        endif
        do 190 n=1,nospec
          do 180 kf=kf1,kf2
             dsigma = sigfin(kf) - sigfin(kf+1)
             do 150 j=2,nyf-1
              jc = jyfb(ifine) + (j-2)/nhf(ifine)
              do 140 i=2,nxf-1
                vfac = 1.e6*delfx(j,ifine)*delfy(ifine)*dsigma
                mfac = pstrf(i,j)/(10.*grav*amolwt)
                ic = ixfb(ifine) + (i-2)/nhf(ifine)
                if (idiag .ne. 0) then
c                  if (ic .eq. 44 .and. jc .eq. 16 .and. n .eq. 1) then
                  if (i .eq. 4 .and. j .eq. nyf/2 .and. n .eq. 15) then
                    write(*,*) ' Sum loop, conc, concf: ',
     &                     conc(ic,jc,n,k), concf(i,j,n,kf)
                    write(*,*) ' vol, delfx, delfy, dsigma, sigmaf:',
     &                     vfac*mfac, delfx(j,ifine), delfy(ifine),
     &                     dsigma, sigfin(kf), sigfin(kf+1)
                    write(*,*) ' ic,jc, i,j, k, kf: ',
     &                    ic, jc, i, j, k, kf
                  endif
                endif
                conc(ic,jc,n,k) = conc(ic,jc,n,k)
     &                  + concf(i,j,n,kf)*vfac*mfac
                if (ldepn .and. kf.eq.1) then
                  dposit(ic,jc,n) = dposit(ic,jc,n) + 
     &                  delfx(j,ifine)*delfy(ifine)*dpositf(i,j,n)
                endif
                if (lwet .and. kf.eq.1) then
                  wposit(ic,jc,n) = wposit(ic,jc,n) + 
     &                  delfx(j,ifine)*delfy(ifine)*wpositf(i,j,n)
                endif
  140         continue
  150       continue
  180     continue
  190   continue
  200 continue
c
c  recalculate conc as concentration instead of mass
c
      do 300 k=1,nlayer
        dsigma = sigma(k) - sigma(k+1)
        do 290 n=1,nospec
          do 280 j=jyfb(ifine),jyfe(ifine)
            do 270 i=ixfb(ifine),ixfe(ifine)
              vfac = 1.e6*deltax(j)*deltay*dsigma
              mfac = pstar(i,j)/(10.*grav*amolwt)
              if (idiag .ne. 0) then
c                if (i .eq. 44 .and. j .eq. 16 .and. n .eq. 1) then
                if (i .eq. 54 .and. j .eq. 42 .and. n .eq. 15) then
                  write(*,*) ' Avg loop, conc: ',
     &                   conc(i,j,n,k)
                  write(*,*) ' vol, deltax, deltay, dsigma, sigma:',
     &                   vfac*mfac, deltax(j), deltay,
     &                   dsigma, sigma(k), sigma(k+1)
                  write(*,*) ' i,j, k: ',
     &                   i, j, k
                endif
              endif
              conc(i,j,n,k) = conc(i,j,n,k)/vfac/mfac
              if (ldepn .and. k.eq.1) then
                dposit(i,j,n) = dposit(i,j,n)/(deltax(j)*deltay)
              endif
              if (lwet .and. k.eq.1) then
                wposit(i,j,n) = wposit(i,j,n)/(deltax(j)*deltay)
              endif
  270       continue
  280     continue
  290   continue
  300 continue
c
      return
      end
