c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine fmsflo(nxf,nyf,nzf,delfx,delfy,sigmaf,cncfa,ufa,vfa,
     &                  pstarf,bcf,deltf,ifine)
c
c-----ATDM v1.00
c
c     Calculates mass flux across fine-grid region boundaries
c
c-----95/06/15:
c     -from UAM-V Revision 1.2  94/02/10  20:56:12  tcm
c     -modified calculation of fbcflo variables for ppm rather than umol/m3
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c-----95/09/14:
c     -fixed some errors in code
c
c     Input arguments:
c               nxf:            number of fine grid cells in x-direction    
c               nyf:            number of fine grid cells in y-direction    
c               nzf:            number of fine grid layers
c               delfx:          fine cell size in x-direction (km)
c               delfy:          fine cell size in y-direction (km)
c               sigmaf:         fine grid sigma structure
c               cncfa:          array of fine grid concentrations (ppm)
c               ufa:            array of x-component windspeeds (km/min) 
c               vfa:            array of y-component windspeeds (km/min)
c               pstarf:         array of fine grid P* (mb)
c               bcf:            array of boundary concentrations (ppm)
c               deltf:          fine grid time step (min)
c               ifine:          fine grid index
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'balanc.cmd'
      include 'advec.cmd'
c
      dimension cncfa(nxf,nyf,nospec,nzf), sigmaf(nzf+1),
     &          ufa(nxf,nyf,nzf), vfa(nxf,nyf,nzf)
      dimension delfx(nyf), pstarf(nxf,nyf)
      dimension bcf(mxfxy,4,mxspec,mxfz,maxfin)
      real mfac
      data grav /9.8/, amolwt /28.8/
c
c  total fluxes across lateral boundaries for all layers
c
      do 200 k=1,nzf
        dsigma = sigmaf(k) - sigmaf(k+1)
        do 150 j=2,nyf-1
c
c         calculate boundary inflow and outflow for west bnd
c
          i = 2
          bfac = 1.e6*deltf*delfy*dsigma
          if (lstagr) then
            wb = ufa(i-1,j,k)
          else
            wb = .5*(ufa(i-1,j,k)+ufa(i,j,k))
          endif
          do 120 l=1,nospec
            if (wb .gt. 0.) then
              mfac = pstarf(i-1,j)/(10.*grav*amolwt)
              cc = bcf(j,4,l,k,ifine)
              fbcflo(l,1,4,ifine) = fbcflo(l,1,4,ifine) + 
     &                              wb*cc*bfac*mfac
            else
              mfac = pstarf(i,j)/(10.*grav*amolwt)
              cc = cncfa(i,j,l,k)
              fbcflo(l,2,4,ifine) = fbcflo(l,2,4,ifine) - 
     &                              wb*cc*bfac*mfac
            endif
  120     continue
c
c         calculate boundary inflow and outflow for east bnd
c
          i = nxf - 1
          bfac = 1.e6*deltf*delfy*dsigma
          if (lstagr) then
            wb = ufa(i,j,k)
          else
            wb = .5*(ufa(i,j,k)+ufa(i+1,j,k))
          endif
          do 140 l=1,nospec
            if (wb .lt. 0.) then
              mfac = pstarf(i+1,j)/(10.*grav*amolwt)
              cc = bcf(j,3,l,k,ifine)
              fbcflo(l,1,3,ifine) = fbcflo(l,1,3,ifine) - 
     &                              wb*cc*bfac*mfac
            else
              mfac = pstarf(i,j)/(10.*grav*amolwt)
              cc = cncfa(i,j,l,k)
              fbcflo(l,2,3,ifine) = fbcflo(l,2,3,ifine) + 
     &                              wb*cc*bfac*mfac
            endif
  140     continue
  150   continue
c
        do 180 i=2,nxf-1
c
c       calculate boundary inflow and outflow for south bnd
c
          j = 2
          bfac = 1.e6*deltf*delfx(j)*dsigma
          if (lstagr) then
            wb = vfa(i,j-1,k)
          else
            wb = .5*(vfa(i,j,k)+vfa(i,j-1,k))
          endif
          do 160 l=1,nospec
            if (wb .gt. 0.) then
              mfac = pstarf(i,j-1)/(10.*grav*amolwt)
              cc = bcf(i,2,l,k,ifine)
              fbcflo(l,1,2,ifine) = fbcflo(l,1,2,ifine) + 
     &                              wb*cc*bfac*mfac
            else
              mfac = pstarf(i,j)/(10.*grav*amolwt)
              cc = cncfa(i,j,l,k)
              fbcflo(l,2,2,ifine) = fbcflo(l,2,2,ifine) - 
     &                              wb*cc*bfac*mfac
            endif
  160     continue
c
c         calculate boundary inflow and outflow for north bnd
c
          j = nyf - 1
          bfac = 1.e6*deltf*delfx(j)*dsigma
          if (lstagr) then
            wb = vfa(i,j,k)
          else
            wb = .5*(vfa(i,j,k)+vfa(i,j+1,k))
          endif
          do 170 l=1,nospec
            if (wb .lt. 0.) then
              mfac = pstarf(i,j+1)/(10.*grav*amolwt)
              cc = bcf(i,1,l,k,ifine)
              fbcflo(l,1,1,ifine) = fbcflo(l,1,1,ifine) - 
     &                              wb*cc*bfac*mfac
            else
              mfac = pstarf(i,j)/(10.*grav*amolwt)
              cc = cncfa(i,j,l,k)
              fbcflo(l,2,1,ifine) = fbcflo(l,2,1,Ifine) + 
     &                              wb*cc*bfac*mfac
            endif
  170     continue
  180   continue
  200 continue
c
c  calculate flow through top
c  NOTE: vertical wind at top = 0, tpflow set to 0
c
c     vf1 = deltf*delfy*1000.
c     do 300 j=2,nyf-1
c       vf2 = vf1*delfx(j)
c       do 290 i=2,nxf
          do 280 l=1,nospec
c           vfac = -etafa(i,j,nlayer)*vf2
c           if (vfac .ge. 0.) then
c             ftpflo(l,2,ifine) = ftpflo(l,2,ifine) 
c    &                            + vfac*cncfa(i,j,l,nlayer)
              ftpflo(l,2,ifine) = 0.
c           else
c             ftpflo(l,1,ifine) = ftpflo(l,1,ifine) - vfac*caloft(l)
              ftpflo(l,1,ifine) = 0.
c           endif
  280     continue
c 290   continue
c 300 continue
c
      return
c
c  formats
c
 2000 format(//,1x,'Mass flux summary for ',f8.2,i8,':',/)
 2010 format(5x,'Species = ',a10,8x,'North')
 2020 format(25x,'In   ',1pe10.3,/,25x,'Out  ',e10.3,/,
     &       25x,'Net  ',e10.3,/)
 2030 format(12x,'West',17x,'Top',16x,'East',14x,'Surface emissions',
     &       3x,1pe10.3,/,5x,'In   ',e10.3,5x,'In   ',e10.3,5x,'In   ',
     &       e10.3,10x,'Elevated emissions  ',e10.3,/,
     &       5x,'Out  ',e10.3,5x,'Out  ',e10.3,5x,'Out  ',e10.3,10x,
     &       'Deposition',10x,e10.3,/,
     &       5x,'Net  ',e10.3,5x,'Net  ',e10.3,5x,'Net  ',e10.3,/,
     &       70x,'Total mass',10x,e10.3,/,33x,'South')
      end
