c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine getqar(nxd,nyd,nx,ny,nsp,iunit,time,ndate,
     $                  emtim1,iemdy1,emtim2,iemdy2,kaemit,
     $                  katol,kaxyl,qa,ieof)
c
c-----ATDM v1.00
c
c     Reads emissions for a specified time interval off a UAM type
c     emissions file and returns parameters for main routine
c
c-----95/04/21:
c     -from UAM-V Revision 1.2  94/02/10  20:56:44  tcm
c     -made provisions for emissions of gasses (mol/hr) and PM (g/hr):
c      convert all input emission rates to mol/min
c     -added kaemit as a passed argument
c-----95/10/30:
c     -added dtdata factor to emission rates in the case that they
c      are in terms of n-hourly rates rather than hourly
c
c     Input arguments:
c               nxd:            maximum number of grid cells in x-direction 
c               nyd:            maximum number of grid cells in y-direction
c               nx:             actual number of grid cells in x-direction
c               ny:             actual number of grid cells in y-direction
c               nsp:            number of emitted species
c               iunit:          area source emissions file unit number
c               time:           model time
c               ndate:          model date
c               kaemit:         array to map emitted species to model species
c     Output arguments
c               emtim1:         beginning time from emissions file
c               iemdy1:         beginning date from emissions file
c               emtim2:         ending time from emissions file
c               iemdy2:         ending date from emissions file
c               qa:             area emissions rate (mol/min)
c               ieof:           end of file error flag
c     Files read:
c               iunit:          area source emissions file
c     Files written:
c               6:              standard output file unit (*)
c     Routines called:
c               jtocal    caltoj
c     Called by:
c               atdm      setup
c
      include 'parchm.cmd'
      include 'oneday.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      dimension qa(nxd,nyd,nsp), kaemit(nsp)
      character*4 mdum(10)
c
      fact = 1./datadt
      ieof = 0
c
c  find correct time and read emissions
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (ndate .gt. 99999) then
        if (iemdy1 .lt. 99999) then
          call jtocal(iemdy1,id1)
          call jtocal(iemdy2,id2)
        else
          id1 = iemdy1
          id2 = iemdy2
        endif
      else
        if (iemdy1 .gt. 99999) then
          call caltoj(iemdy1,id1)
          call caltoj(iemdy2,id2)
        else
          id1 = iemdy1
          id2 = iemdy2
        endif
      endif
      tchk = emtim1*100. + 2400.*(id1-ndate)
      tcend = emtim2*100. + 2400.*(id2-ndate)
c
      if (time .ge. tchk-1. .and. time .le. tcend-1.) return
c
   30 read (iunit,end=900) iemdy1, emtim1, iemdy2, emtim2
c
      write (*,*) 'in getqar',iemdy1, emtim1, iemdy2, emtim2
      write(*,'(a,i8,f10.2,a,i8,f10.2,a,i4)')
     &     ' Read interval ',iemdy1, emtim1, ' to ', iemdy2, emtim2,
     &     ' on unit ', iunit
      if (l1eday) then
        if ( iemdy1 .gt. 99999 ) then
          call caltoj(iemdy1,iid1)
        else
          iid1=iemdy1
        endif
        if ( iemdy2 .gt. 99999 ) then
          call caltoj(iemdy2,iid2)
        else
          iid2=iemdy2
        endif
        if ( ndate .gt. 99999 ) then
          call caltoj(ndate,nnd1)
        else
          nnd1=ndate
        endif
        ndif = iid2 - iid1
        iid1 = nnd1
        iid2 = iid1 + ndif
        iemdy1 = nnd1
        iemdy2 = iemdy1 + ndif
c        ndif = iemdy2 - iemdy1
c        iemdy1 = ndate
c        iemdy2 = iemdy1 + ndif
      endif
c
      do 200 l=1,nsp
        read(iunit,end=900) nseg,(mdum(ii),ii=1,10),
     &                      ((qa(i,j,l),i=1,nx),j=1,ny)
c
c  convert emission rates from moles/n-hour to moles/min for gasses
c                     and from g/n-hour to moles/min for PM
c
        ll = kaemit(l)
        if (ll .eq. kvoc) kkv = l
        if (ll .eq. kcarb) kkc = l
        if (ll.eq.0) goto 200
        lll = jind(ll)
c       write(*,*) 'In getqar, l, ll, lll, ngas, molwt(lll): '
c       write(*,*) l, ll, lll, ngas, molwt(lll)
        do 50 j=1,ny
          do 40 i=1,nx
            if (ll.le.ngas) then
               qa(i,j,l) = qa(i,j,l)*fact
            else
               qa(i,j,l) = qa(i,j,l)*fact/molwt(lll)
            endif
   40     continue
   50   continue
  200 continue
c
c  for soa chem, calculate aromatic fraction
c
      if (katol .gt. 0 .and. kaxyl .gt. 0) then
        do j=1,ny
          do i=1,nx
            if (qa(i,j,kkv) .gt. 0. .or. qa(i,j,kkc) .gt. 0.) then
              qa(i,j,katol) = (7.*qa(i,j,katol)+8.*qa(i,j,kaxyl))*fact/
     &                     (qa(i,j,kkv)+qa(i,j,kkc))
              if (qa(i,j,katol) .gt. 1.) qa(i,j,katol) = 1.
              if (qa(i,j,katol) .lt. 0.) qa(i,j,katol) = 0.
            endif
          enddo
        enddo
      endif
c
c  check time and if not correct, go back for another interval
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (ndate .gt. 99999) then
        if (iemdy1 .lt. 99999) then
          call jtocal(iemdy1,id1)
          call jtocal(iemdy2,id2)
        else
          id1 = iemdy1
          id2 = iemdy2
        endif
      else
        if (iemdy1 .gt. 99999) then
          call caltoj(iemdy1,id1)
          call caltoj(iemdy2,id2)
        else
          id1 = iemdy1
          id2 = iemdy2
        endif
      endif
      tchk = emtim1*100. + 2400.*(id1-ndate)
      tcend = emtim2*100. + 2400.*(id2-ndate)
      if (time .ge. tcend) go to 30
      if (time .lt. tchk-1.) then
        write(*,*) ' Time error on EMISSIONS file',
     &             ' Current time: ',time, ndate,
     &             ' File time: ',emtim2,iemdy2
        stop
      endif
c
      return
c
  900 ieof = 1
      return
c
      end
