c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine hedibc(nosp, nxsz, nysz, iunit, nspc, kspc, mfil)
c
c-----ATDM v1.00
c
c     Reads header of UAM initial conditions file and
c     fills in parameters
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  20:57:01  tcm
c     -removed code specific to FORM and HONO
c
c     Input arguments:
c               nosp:           number of modeled species    
c               nxsz:           number of coarse grid cells in x-direction
c               nysz:           number of coarse grid cells in y-direction
c               iunit:          initial condition file unit number
c     Output arguments:
c               nspc:           number of species from IC file
c               kspc:           array to map IC species to modeled species
c               mfil:           name of IC file (INSTANT or AIRQUALITY)
c     Files read:
c               iunit:          initial condition file
c     Files written:
c               6:              standard output file unit (*)
c     Called by:
c               setup
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comchem.cmd'
c
      dimension kspc(mxibc)
c
      character*4 msp(10,mxibc), note(60), ifile(10) 
      character*4 mdum(10)
      character*10 m1, m2
      character*10 mfil
c
      read(iunit) ifile, note, nseg, nflspc, idate, begtim, 
     &            jdate, endtim
      write(*,'(a,i2,a,10a1)') ' File on unit ',iunit,' is ',ifile
c
c  pass file name back to calling routine in character variable
c
      write(mfil,'(10a1)') ifile
c
      if (nflspc .gt. mxibc) then
        write(*,*) 'NUMBER OF SPECIES ON INPUT FILE (',nflspc,')',
     &             ' IS GREATER THAN MAX ALLOWED (',mxibc,').'
        stop
      endif
      read  (iunit) orgx, orgy, izone, utmx, utmy, deltax, deltay,
     &          nx, ny, nz, nzlowr, nzuppr, htsur, htlow, htupp
      if (nx .ne. nxsz) then
        write(*,*) 'NUMBER OF X CELLS ON INPUT FILE (',nx,')',
     &             ' IS INCORRECT. (',nxsz,').'
        stop
      endif
      if (ny .ne. nysz) then
        write(*,*) 'NUMBER OF Y CELLS ON INPUT FILE (',ny,')',
     &             ' IS INCORRECT. (',nysz,').'
        stop
      endif
      read(iunit) nxx,nyy,nxcell,nycell
      read(iunit) ((msp(m,l),m=1,10),l=1,nflspc)
c
c  find each species on file in standard species list
c  (0 will mean a species on file that is not needed.)
c
      do 100 l=1,nflspc
        write(m1,'(10a1)') (msp(i,l),i=1,10)
        do 80 lint=1,nosp
          write(m2,'(10a1)') (mspec(i,lint),i=1,10)
          if (m1 .eq. m2) then
            kspc(l) = lint
            write(*,'(a,i3,1x,a,a,10a1,a,i3,1x,a,a)') ' Species ',l,m1,
     &            ' on ',ifile,' file = species ',lint,m2,' internally.'
            go to 100
          endif
   80   continue
        kspc(l) = 0
  100 continue
c
      nspc = nflspc
      write (*,*) 'end of hedibc'
      return
      end
