c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine hedpts(nosp, nxsz, nysz, iunit, nsem, kespc)
c
c-----ATDM v1.00
c
c     Reads header of UAM point source emissions file and
c     fills in parameters for main routine
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  20:57:06  tcm
c     -removed code specific to FORM and HONO
c     -removed PiG-related code
c     -introduced new variables PTSRCX & PTSRCY to carry point source
c      grid coordinates: input PLON (formerly PLONG) and PLAT now
c      carry UTM (LCART=T) or lat/lon coordinates (LCART=F).  This allows
c      for removal of call to subroutine CONVRT from several routines
c-----95/10/04:
c     -temporarily coded to handle input stack coords in (lat,lon)*1000
c-----95/12/12:
c     -Removed above temporary fix
c
c     Input arguments:
c               nosp:           number of modeled species   
c               nxsz:           number of coarse grid cells in x-direction 
c               nysz:           number of coarse grid cells in y-direction
c               iunit:          point source file unit number
c     Output arguments:
c               nsem:           number of emitted species
c               kespc:          array to map emitted species to modeled species
c     Files read:
c               iunit:          point source file
c     Files written:
c               6:              standard output file unit (*)
c     Called by:
c               atdm      setup
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'stack.cmd'
      include 'comchem.cmd'
      include 'latlon.cmd'
      include 'com3.cmd'
c
      dimension kespc(mxemit)
c
      character*4 memsp(10,mxemit), note(60), ifile(10) 
      character*10 m1, m2
      nplm = 0
c
      read(iunit) ifile, note, nseg, nemspc, idate, begtim, 
     &            jdate, endtim
      write(m1,'(10a1)') ifile
      if (m1 .ne. 'PTSOURCE  ') then
        write(*,'(a,i2,a,a,/,a)') ' File on unit ',iunit,' is ',m1,
     &           ' It should be PTSOURCE.'
        stop
      endif
      if (nemspc .gt. mxemit) then
        write(*,*) 'NUMBER OF SPECIES ON PTSOURCE FILE (',nemspc,')',
     &             ' IS GREATER THAN MAX ALLOWED (',mxemit,').'
        stop
      endif
      read  (iunit) orgx, orgy, izone, utmx, utmy, deltax, deltay,
     &          nx, ny, nz, nzlowr, nzuppr, htsur, htlow, htupp
      if (nx .ne. nxsz) then
        write(*,*) '*** WARNING ***' 
        write(*,*) 'NUMBER OF X CELLS ON PTSOURCE FILE (',nx,')',
     &             ' IS INCORRECT. (',nxsz,').'
      endif
      if (ny .ne. nysz) then
        write(*,*) '*** WARNING ***' 
        write(*,*) 'NUMBER OF Y CELLS ON PTSOURCE FILE (',ny,')',
     &             ' IS INCORRECT. (',nysz,').'
      endif
      read(iunit) nxx,nyy,nxcell,nycell
      read(iunit) ((memsp(m,l),m=1,10),l=1,nemspc)
      write (*,*) 'all species'
 1001 format (i4,5x,10a1)
c      do l = 1, nemspc
c         write (*,1001) l, (memsp(m,l), m=1, 10)
c      end do
c      write (*,*) 'nosp'
c      do lint = 1, nosp
c         write (*,1001) lint, (mspec(i,lint), i=1, 10)
c      end do
c
c  find each species on emissions file in standard species list
c  (0 will mean a species on emissions file that is not needed.)
c
      do 100 l=1,nemspc
        write(m1,'(10a1)') (memsp(i,l),i=1,10)
        do 80 lint=1,nosp
          write(m2,'(10a1)') (mspec(i,lint),i=1,10)
          if (m1 .eq. m2) then
            kespc(l) = lint
            write(*,'(a,i3,1x,a,a,i3,1x,a,a)') ' Species ',l,m1,
     &            ' on PTSOURCE file = species ',lint,m2,' internally.'
            go to 100
          endif
   80   continue
        kespc(l) = 0
  100 continue
c
      nsem = nemspc
c
c  read and check number of points
c
      read(iunit) idum, nopts
      write (*,*) 'hedpts nopts when read=', nopts
      if (nopts .gt. mxsrc) then
        write(*,'(1x,i5,a,/,1x,a,i5)') nopts, 
     &                          ' is larger than the maximum',
     &                          ' number of allowed sources: ',mxsrc
        stop
      endif
c
c  read location and stack parameters
c
      read(iunit) (plon(n), plat(n), stkht(n), stkd(n),
     &            temps(n), vexit(n),n=1,nopts)
c
c  convert location to grid coordinates, convert vexit from m/hr to m/s 
c
      do 120 n=1,nopts
        if(lcart)then
          ptsrcx(n) = (plon(n)*.001-xorg)/dlong
          ptsrcy(n) = (plat(n)*.001-yorg)/dlat
        else
          ptsrcx(n) = (plon(n)-wlong)/dlong
          ptsrcy(n) = (plat(n)-slat)/dlat
        endif
        vexit(n) = vexit(n)/3600.
  120 continue
c      do l = 1, mxemit
c      write (*,*) l, kespc(l)
c      end do
      write (*,*) 'hedpts nopts at the end=', nopts
      return
      end
