c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine htfnew(nxf,nyf,nzf,deltht,pmodtop,sigmaf,psoldf,
     &                  psnewf,temprf,ch2of,psfcf,pstarf,presf,htf)
c
c-----ATDM v1.00
c
c     For current fine grid timestep, updates values of fine grid surface
c     pressure, P*, layer pressure, and layer interface heights for ensuing 
c     calculations
c
c-----95/06/15:
c     C Emery SAI
c
c     Input arguments:
c               nxf:            number of fine grid cells in x-direction    
c               nyf:            number of fine grid cells in y-direction    
c               nzf:            number of fine grid layers
c               deltht:         elapsed time since last met update (min)
c               pmodtop:        pressure at model top (mb)
c               sigmaf:         array of fine grid sigma levels
c               psoldf:         array of old surface pressure (mb)
c               psnewf:         array of new surface pressure (mb)
c               temprf:         array of temperature (K)
c               ch2of:          array of water vapor concentration (kg/kg)
c     Output arguments:
c               psfcf:          current surface pressure (mb)
c               pstarf:         current value of P* (mb)
c               presf:          current 3-D pressure (mb)
c               htf:            current layer interface heights (m)
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'com3.cmd'
      dimension sigmaf(nzf+1)
      dimension psoldf(nxf,nyf),psnewf(nxf,nyf),temprf(nxf,nyf,nzf),
     &          ch2of(nxf,nyf,nzf)
      dimension psfcf(nxf,nyf),pstarf(nxf,nyf),presf(nxf,nyf,nzf),
     &          htf(nxf,nyf,nzf)
c
      data eps /0.622/, rd /287./, grav /9.8/
c
      do 90 j=1,nyf
        do 80 i=1,nxf
c
c     Update surface pressure from PSOLDF and PSNEWF
c
          psfcf(i,j) = psoldf(i,j) + 
     &                 deltht*(psnewf(i,j) - psoldf(i,j))/datadt
          pstarf(i,j) = psfcf(i,j) - pmodtop
c
c     Update PRESF and HTF arrays
c
          do 82 k = 1,nzf
            prsupr = sigmaf(k+1)*pstarf(i,j) + pmodtop
            prslwr = sigmaf(k)*pstarf(i,j) + pmodtop
            presf(i,j,k) = (prsupr + prslwr)/2.
            tvirt = temprf(i,j,k)*(eps + ch2of(i,j,k))/
     &              (eps*(1. + ch2of(i,j,k)))
            hbar = rd*tvirt/grav
            if (k.eq.1) then
              htf(i,j,k) = hbar*alog(psfcf(i,j)/prsupr)
            else
              htf(i,j,k) = htf(i,j,k-1) + hbar*alog(prslwr/prsupr)
            endif
   82     continue
c
   80   continue
   90 continue
      return
      end
