c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine maputg (utmx,utmy,izone,alon,alat)
c
c-----ATDM v1.00
c
c     Converts UTM coordinates to latitude and longitude
c     This is a modification of the us geological survey
c     program no j380  (topographic division)
c     G Lundberg SAi 4/79
c
c-----95/06/01:
c     -from Revision 1.2  94/02/10  20:58:11  tcm
c
c     Input arguments:
c               utmx:           UTM easting (km)
c               utmy:           UTM northing (km)
c               izone:          UTM zone number
c     Output arguments:
c               alon:           longitude (degrees, negative in western hem)
c               alat:           latitude (degrees)
c     Routines called:
c               vvgtu1 
c     Called by:
c               setup
c
      dimension b(12)
      common /vvutm/ a(16)
c
      x = utmx*1000.
      y = utmy*1000.
c
c     *** compute conversion coefficients if needed
c
      call vvgtu1
c
c     *** compute central meridian in seconds from izone input
c
   10 utz = 30.0 - iabs (izone)
      a(9) = ((utz*6.0) + 3.0) * 3600.0
c
c     *** convert utm coordinates to geodetic
c
      b(9)=((a(5)-x)*1.0e-6)/a(8)
   30 b(10)=((y-a(6))/a(8)+a(7))/a(10)
   40 sinw=sin(b(10))
      cosw=cos(b(10))
      b(12)=cosw*cosw
      b(11)=(((a(14)*b(12)+a(13))*b(12)+a(12))*b(12)+a(11))*sinw*cosw
     & +b(10)
      sinw=sin(b(11))
      cosw=cos(b(11))
      rn=sqrt(1.0-a(16)*sinw*sinw)*1.0e6/a(15)
      t=sinw/cosw
      ts=t*t
      b(12)=cosw*cosw
      etas=a(16)*b(12)/(1.0-a(16))
      b(1)=rn/cosw
      b(2)=-t*(1.0+etas)*rn*rn/2.0
      b(3)=-(1.0+2.0*ts+etas)*b(1)*rn*rn/6.0
      b(4)=(((-6.0-etas*9.0)*etas+3.0)*ts+(6.0-etas*3.0)*etas
     & +5.0)*t*rn**4/24.0
      b(5)=((ts*24.0+etas*8.0+28.0)*ts+etas*6.0+5.0)*b(1)*
     & rn**4/120.0
      b(6)=(((etas*45.0-45.0)*ts+etas*162.0-90.0)*ts
     & -etas*107.0-61.0)*t*rn**6/720.0
      b(7)=-(((ts*720.0+1320.0)*ts+662.0)*ts+61.0)*b(1)*rn**6/
     & 5040.0
      b(8)=(((ts*1575.0+4095.0)*ts+3633.0)*ts+1385.0)*t*rn**8/
     & 40320.0
      b(10)=b(9)*b(9)
      slat=((((b(8)*b(10)+b(6))*b(10)+b(4))*b(10)+b(2))*b(10)+b(11))*
     & 206264.80624709
      slon= (((b(7)*b(10)+b(5))*b(10)+b(3))*b(10)+b(1))*b(9)*
     & 206264.80624709 + a(9)
c
c     *** convert seconds of arc to degrees and return
c
   99 alat = slat/3600.
      alon = -slon/3600.
c
      return
      end
