c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine masflo(time,ndate,deltat,lprint)
c
c-----ATDM v1.00
c
c     Performs a simple calculation of mass flux across region boundaries
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  20:58:19  tcm
c     -removed calculation and output of PiG mass flux variables
c-----95/06/07:
c     -modified calculation of bcflo variables for ppm rather than umol/m3
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c     -added data statements for grav and amolwt
c
c     Input arguments:
c               time:           model time   
c               ndate:          model date
c               deltat:         time step (min)
c               lprint:         print flag
c     Files written:
c               8:              diagnostic file
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'balanc.cmd'
      include 'advec.cmd'
c
      logical lprint
      real mfac
      data grav /9.8/, amolwt /28.8/
c
c  if we have reached an output time, print the mass fluxes and
c  reset them to zero.
c
      if (lprint) then
c
c  add this interval's fluxes into cumulative arrays
c
        do 350 l=1,nospec
          eflcu(l) = eflcu(l) + eflow(l)
          ptflcu(l) = ptflcu(l) + ptflow(l)
          dpflcu(l) = dpflcu(l) + dpflow(l)
          do 340 n=1,2
            tpflcu(l,n) = tpflcu(l,n) + tpflow(l,n)
            do 330 k=1,4
              bcflcu(l,n,k) = bcflcu(l,n,k) + bcflow(l,n,k)
  330       continue
  340     continue
  350   continue
c
c  print mass flux summary (only for species to appear on average file)
c
        write(8,2000) time, ndate
        do 400 l=1,navspc,2
          linspc = kavspc(l)
          if (l .lt. navspc) lins2 = kavspc(l+1)
          if (l .eq. navspc) then
            write(8,2010) mavspc(l)
            fnet = bcflow(linspc,1,1) - bcflow(linspc,2,1)
            write(8,2020) bcflow(linspc,1,1), eflow(linspc),
     &                    bcflow(linspc,2,1), ptflow(linspc),
     &                    fnet, dpflow(linspc)
            fnetw = bcflow(linspc,1,4) - bcflow(linspc,2,4)
            fnett = tpflow(linspc,1) - tpflow(linspc,2)
            fnete = bcflow(linspc,1,3) - bcflow(linspc,2,3)
            write(8,2030) bcflow(linspc,1,4), 
     &                  tpflow(linspc,1), bcflow(linspc,1,3),
     &                  bcflow(linspc,2,4),
     &                  tpflow(linspc,2), bcflow(linspc,2,3),
     &                  fnetw, fnett, fnete
            fnet = bcflow(linspc,1,2) - bcflow(linspc,2,2)
            write(8,2035) bcflow(linspc,1,2), bcflow(linspc,2,2), 
     &                    tmas(linspc), fnet
          else
            write(8,2110) mavspc(l), mavspc(l+1)
            fnet = bcflow(linspc,1,1) - bcflow(linspc,2,1)
            fnet2 = bcflow(lins2,1,1) - bcflow(lins2,2,1)
            write(8,2120) bcflow(linspc,1,1), eflow(linspc), 
     &            bcflow(lins2,1,1), eflow(lins2),
     &            bcflow(linspc,2,1), ptflow(linspc),
     &            bcflow(lins2,2,1), ptflow(lins2),
     &            fnet, dpflow(linspc), fnet2, dpflow(lins2)
            fnetw = bcflow(linspc,1,4) - bcflow(linspc,2,4)
            fnett = tpflow(linspc,1) - tpflow(linspc,2)
            fnete = bcflow(linspc,1,3) - bcflow(linspc,2,3)
            fnetw2 = bcflow(lins2,1,4) - bcflow(lins2,2,4)
            fnett2 = tpflow(lins2,1) - tpflow(lins2,2)
            fnete2 = bcflow(lins2,1,3) - bcflow(lins2,2,3)
            write(8,2130) bcflow(linspc,1,4), 
     &                  tpflow(linspc,1), bcflow(linspc,1,3),
     &                  bcflow(lins2,1,4), 
     &                  tpflow(lins2,1), bcflow(lins2,1,3),
     &                  bcflow(linspc,2,4),
     &                  tpflow(linspc,2), bcflow(linspc,2,3),
     &                  bcflow(lins2,2,4),
     &                  tpflow(lins2,2), bcflow(lins2,2,3),
     &                  fnetw, fnett, fnete,
     &                  fnetw2, fnett2, fnete2
            fnet = bcflow(linspc,1,2) - bcflow(linspc,2,2)
            fnet2 = bcflow(lins2,1,2) - bcflow(lins2,2,2)
            write(8,2135) bcflow(linspc,1,2), bcflow(lins2,1,2),
     &            bcflow(linspc,2,2), tmas(linspc), bcflow(lins2,2,2), 
     &            tmas(lins2), fnet, fnet2
          endif
          write(8,2040)
  400   continue
c
c  zero out flux arrays for next interval
c
        do 430 l=1,nospec
          eflow(l) = 0.
          ptflow(l) = 0.
          dpflow(l) = 0.
          do 420 n=1,2
            tpflow(l,n) = 0.
            do 410 k=1,4
              bcflow(l,n,k) = 0.
  410       continue
  420     continue
  430   continue
      endif
c
c  total fluxes across lateral boundaries for all layers
c
      do 200 k=1,nlayer
        dsigma = sigma(k) - sigma(k+1)
        do 150 j=1,noy
c
c       calculate boundary inflow and outflow for west bnd
c
          if (istr(j) .gt. nox) go to 150
          i = istr(j) + 1
          bfac = 1.e6*deltat*deltay*dsigma
          if (lstagr) then
            wb = u(i-1,j,k)
          else
            wb = .5*(u(i-1,j,k)+u(i,j,k))
          endif
          do 120 l=1,nospec
            if (wb .gt. 0.) then
              mfac = pstar(i-1,j)/(10.*grav*amolwt)
              cc = bc(j,4,l,k)
              bcflow(l,1,4) = bcflow(l,1,4) + wb*cc*bfac*mfac
            else
              mfac = pstar(i,j)/(10.*grav*amolwt)
              cc = conc(i,j,l,k)
              bcflow(l,2,4) = bcflow(l,2,4) - wb*cc*bfac*mfac
            endif
  120     continue
c
c             calculate boundary inflow and outflow for east bnd
c
          i = iend(j) - 1
          bfac = 1.e6*deltat*deltay*dsigma
          if (lstagr) then
            wb = u(i,j,k)
          else
            wb = .5*(u(i,j,k)+u(i+1,j,k))
          endif
          do 140 l=1,nospec
            if (wb .lt. 0.) then
              mfac = pstar(i+1,j)/(10.*grav*amolwt)
              cc = bc(j,3,l,k)
              bcflow(l,1,3) = bcflow(l,1,3) - wb*cc*bfac*mfac
            else
              mfac = pstar(i,j)/(10.*grav*amolwt)
              cc = conc(i,j,l,k)
              bcflow(l,2,3) = bcflow(l,2,3) + wb*cc*bfac*mfac
            endif
  140     continue
  150   continue
c
c             calculate boundary inflow and outflow for south bnd
c
        do 180 i=1,nox
          if (jstr(i) .gt. noy) go to 180
          j = jstr(i) + 1
          bfac = 1.e6*deltat*deltax(j)*dsigma
          if (lstagr) then
            wb = v(i,j-1,k)
          else
            wb = .5*(v(i,j,k)+v(i,j-1,k))
          endif
          do 160 l=1,nospec
            if (wb .gt. 0.) then
              mfac = pstar(i,j-1)/(10.*grav*amolwt)
              cc = bc(i,2,l,k)
              bcflow(l,1,2) = bcflow(l,1,2) + wb*cc*bfac*mfac
            else
              mfac = pstar(i,j)/(10.*grav*amolwt)
              cc = conc(i,j,l,k)
              bcflow(l,2,2) = bcflow(l,2,2) - wb*cc*bfac*mfac
            endif
  160     continue
c
c       calculate boundary inflow and outflow for north bnd
c
          j = jend(i) - 1
          bfac = 1.e6*deltat*deltax(j)*dsigma
          if (lstagr) then
            wb = v(i,j,k)
          else
            wb = .5*(v(i,j,k)+v(i,j+1,k))
          endif
          do 170 l=1,nospec
            if (wb .lt. 0.) then
              mfac = pstar(i,j+1)/(10.*grav*amolwt)
              cc = bc(i,1,l,k)
              bcflow(l,1,1) = bcflow(l,1,1) - wb*cc*bfac*mfac
            else
              mfac = pstar(i,j)/(10.*grav*amolwt)
              cc = conc(i,j,l,k)
              bcflow(l,2,1) = bcflow(l,2,1) + wb*cc*bfac*mfac
            endif
  170     continue
  180   continue
  200 continue
c
c  calculate flow through top
c  NOTE: vertical wind at top = 0, tpflow set to 0
c
c     vf1 = deltat*deltay*1000.
c     do 300 j=1,noy
c       if (istr(j) .gt. nox) go to 300
c       vf2 = vf1*deltax(j)
c       do 290 i=istr(j),iend(j)
          do 280 l=1,nospec
c           vfac = -eta(i,j,nlayer)*vf2
c           if (vfac .ge. 0.) then
c             tpflow(l,2) = tpflow(l,2) + vfac*conc(i,j,l,nlayer)
              tpflow(l,2) = 0.
c           else
c             tpflow(l,1) = tpflow(l,1) - vfac * caloft(l)
              tpflow(l,1) = 0.
c           endif
  280     continue
c 290   continue
c 300 continue
c
      return
c
c  formats
c
 2000 format(//,1x,'mass flux summary in moles for coarse grid at',
     &       f8.2,i8,':',/)
 2010 format(1x,'spec = ',a10,2x,'north')
 2020 format(13x,'in  ',1pe10.3,15x,'srf emiss ',e10.3,/,
     &       13x,'out ',e10.3,15x,'elv emiss ',e10.3,/,
     &       13x,'net ',e10.3,15x,'depostn   ',e10.3,/)
 2030 format(8x,'west',8x,'top',8x,'east',/
     &       1x,'in  ',1pe10.3,2x,e10.3,2x,e10.3,/,
     &       1x,'out ',e10.3,2x,e10.3,2x,e10.3,/,
     &       1x,'net ',e10.3,2x,e10.3,2x,e10.3,//,
     &       20x,'south')
 2035 format(13x,'in  ',1pe10.3,/,
     &       13x,'out ',e10.3,15x,'tot mass  ',e10.3,/,
     &       13x,'net ',e10.3,e10.3,/)
 2040 format(1x,126(1h-))
 2110 format(1x,'spec = ',a10,2x,'north',38x,'| ','spec = ',
     &       a10,2x,'north')
 2120 format(13x,'in  ',1pe10.3,15x,'srf emiss ',e10.3,1x,'|',
     &       13x,'in  ',e10.3,15x,'srf emiss ',e10.3,/,
     &       13x,'out ',e10.3,15x,'elv emiss ',e10.3,1x,'|',
     &       13x,'out ',e10.3,15x,'elv emiss ',e10.3,/,
     &       13x,'net ',e10.3,15x,'depostn   ',e10.3,1x,'|',
     &       13x,'net ',e10.3,15x,'depostn   ',e10.3,/,63x,'|')
 2130 format(8x,'west',8x,'top',8x,'east',28x,
     &       '|',8x,'west',8x,'top',8x,'east',
     &       /,1x,'in  ',1pe10.3,2x,e10.3,2x,e10.3,24x,'|',
     &       1x,'in  ',e10.3,2x,e10.3,2x,e10.3,/,
     &       1x,'out ',e10.3,2x,e10.3,2x,e10.3,24x,'|',
     &       1x,'out ',e10.3,2x,e10.3,2x,e10.3,/,
     &       1x,'net ',e10.3,2x,e10.3,2x,e10.3,24x,'|',
     &       1x,'net ',e10.3,2x,e10.3,2x,e10.3,/,
     &       63x,'|',/,20x,'south',38x,'|',20x,'south')
 2135 format(13x,'in  ',1pe10.3,36x,'|',
     &       13x,'in  ',e10.3,/,
     &       13x,'out ',e10.3,15x,'tot mass  ',e10.3,1x,'|',
     &       13x,'out ',e10.3,15x,'tot mass  ',e10.3,/,
     &       13x,'net ',e10.3,36x,'|',
     &       13x,'net ',e10.3,/,63x,'|')
      end
