c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine pblmod(ws,ta,tsurf,pa,psurf,zs,z0,elinv,ustar,wref)
c
c-----ATDM v1.00
c
c     Calculates u*, monin-obhukov length, and 10-m windspeed
c     based on parameterizations of Louis (bound. lay. met., vol. 17,
c     1979, 187-202)
c
c-----95/06/05:
c     -from UAM-V Revision 1.2  94/02/10  20:58:35  tcm
c     -added surface pressure to argument list, and deleted calculation
c      of surface pressure in code
c
c     Input arguments:
c               ws:             wind magnitude at first layer midpoint (m/s)
c               ta:             temperature at first layer midpoint (k)
c               tsurf:          surface temperature (k)
c               pa:             pressure at first layer midpoint (mb)
c               psurf:          surface pressure (mb)
c               zs:             height at top of first layer (m)
c               z0              surface roughness length (m)
c     Output arguments:
c               ustar:          friction velocity (m/s)
c               elinv:          inverse of m-o length (m^-1)
c               wref:           windspeed at reference (10 m) height (m/s)
c     Called by:
c               surlyr    srlyrf
c
      real lnzz
      data zr /10./, vk /0.4/, pi /3.1415927/, eps /1.e-10/, g /9.8/
      data dpdz /0.1/, gamma /0.286/
c
      za = zs/2.
      theta1 = ta*(1000./pa)**gamma
      thetas = tsurf*(1000./psurf)**gamma
c
c-----calculate temperature gradients, a drag coefficient, and richardson #.
c
      dtheta = theta1 - thetas
      thbar = (thetas + theta1)/2.
      richnum = g*za*dtheta/(thbar*ws**2.+eps)
      lnzz = alog(za/z0)
      lnzzr = alog(zr/z0)
      a2 = (vk/lnzz)**2.
      ar2 = (vk/lnzzr)**2.
c
c-----find richardson number-dependent stability functions fm and fh
c
      if (richnum.lt.0.) then
        cm = 7.4*a2*9.4*sqrt(za/z0)
        cmr = 7.4*ar2*9.4*sqrt(zr/z0)
        ch = 5.3*a2*9.4*sqrt(za/z0)
        fm = 1.-9.4*richnum/(1.+cm*sqrt(abs(richnum))+eps)
        fmr = 1.-9.4*richnum/(1.+cmr*sqrt(abs(richnum))+eps)
        fh = 1.-9.4*richnum/(1.+ch*sqrt(abs(richnum))+eps)
      else
        fm = 1./((1.+4.7*richnum)**2.+eps)
        fmr = fm
        fh = fm
      endif
c
c-----find u*, theta* as f(fm,fh,m,dtheta,a2)
c
      ustr2 = a2*fm*ws**2.
      ustar = sqrt(ustr2)
      thstr = 1.35*a2*ws*dtheta*fh/ustar
c
c-----calculate 10-m wind
c
      wref = sqrt(ustr2/(ar2*fmr))
c
c-----find m-o length
c
      el = ustr2*ta/(vk*g*thstr+eps)
      elinv = 1./(el+eps)
c       
      return
      end
