c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine prtint(time,ndate)
c
c-----ATDM v1.00
c
c     Prints initial concentrations to inst file 
c
c-----95/06/05:
c     -from UAM-V Revision 1.2  94/02/10  20:59:12  tcm
c-----99/04/22:DFG
c     -added loops to write initial oh and o3 coarse concentration
c      when lmoh is true.
c     -added loops to write initial oh fine grid concentration when
c      lmoh is true and numfin >0.
c     -added ko3 and lfohf in passing arguments in calling FGWRIT.
c      in FGWRIT, fine o3 wil be written out to oh file.
c
c     Input arguments:
c               time:           model time    
c               ndate:          model date
c     Files written:
c               51:             instantaneous output concentration file
c               ifinst:         fine grid instantaneous concentration file
c     Routines called:
c               fgwrit
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'com1f.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
c
      tmu2 = time/100.
      tmui2 = tmu2 + .001
      tmui1 = tmu2 - .001
      ndu2 = ndate
      idum = 1
c
c     write out dump files
c
      write(*,*) 'within prtint-coarse grid', ndu2, tmui1, ndu2, tmui2
      write(51) ndu2, tmui1, ndu2, tmui2
      do 4 k=1,nospc1
        do 3 l=1,nlayer
          write(51) idum, (mspec(i,k),i=1,10),
     &              ((conc(i,j,k,l),i=1,nox),j=1,noy)
    3   continue
    4 continue
c
c  Write out oh concentration if necessary. 4/14/99, dfg
c
       if (lmoh) then
          write(iohf) ndu2, tmui1, ndu2, tmui2
          do l=1,nlayer
            write(iohf) idum,(msoh(m),m=1,10),
     &                     ((ohcon(i,j,l),i=1,nox),j=1,noy)
          end do
          do l=1,nlayer
            write(iohf) idum,(msho2(m),m=1,10),
     &                     ((ho2con(i,j,l),i=1,nox),j=1,noy)
          end do
          do l=1,nlayer
            write(iohf) idum,(msno3(m),m=1,10),
     &                     ((no3con(i,j,l),i=1,nox),j=1,noy)
          end do
          do l=1,nlayer
            write(iohf) idum,(msaso2(m),m=1,10),
     &                     ((aso2con(i,j,l),i=1,nox),j=1,noy)
          end do
          do l=1,nlayer
            write(iohf) idum,(mspec(i,kh2o2),i=1,10),
     &                     ((conc(i,j,kh2o2,l),i=1,nox),j=1,noy)
          end do
          do l=1,nlayer
            write(iohf) idum,(mspec(i,ko3),i=1,10),
     &                     ((conc(i,j,ko3,l),i=1,nox),j=1,noy)
          end do
          do l=1,nlayer
            write(iohf) idum,(mspec(i,kpec),i=1,10),
     &                     ((conc(i,j,kpec,l),i=1,nox),j=1,noy)
          end do
       endif
c
c  write out fine grid inst concs
c
       write (*,*) 'within prtint-fine grid',  time, ndate
      if (ifinst .gt. 0) write(ifinst) time, ndate
      if (lmoh .and. ifohf.gt.0) write(ifohf) time,ndate
      do 5 ifine=1,numfin
        if (lmoh) then
          call wrfo3oh(ifohf,nfx(ifine),nfy(ifine),nfz(ifine),
     &                   ohconf(1,1,1,ifine),no3conf(1,1,1,ifine),
     &                   ho2conf(1,1,1,ifine),aso2conf(1,1,1,ifine))
c
c         do k=1,nfz(ifine)
c           write(ifohf) ((ohconf(i,j,k,ifine),i=1,nfx(ifine)),
c    &                   j=1,nfy(ifine))
c         end do
c         do k=1,nfz(ifine)
c           write(ifohf) ((ho2conf(i,j,k,ifine),i=1,nfx(ifine)),
c    &                   j=1,nfy(ifine))
c         end do
c         do k=1,nfz(ifine)
c           write(ifohf) ((no3conf(i,j,k,ifine),i=1,nfx(ifine)),
c    &                   j=1,nfy(ifine))
c         end do
c         do k=1,nfz(ifine)
c           write(ifohf) ((aso2conf(i,j,k,ifine),i=1,nfx(ifine)),
c    &                   j=1,nfy(ifine))
c         end do
        endif

        if (ifinst .gt. 0) then
          call fgwrit(ifinst,nfx(ifine),nfy(ifine),nfz(ifine),nospec,
     &                cncf(1,1,1,ifine),ifohf,ko3,kh2o2,kpec)
        endif
    5 continue
c
c  ---flush causes buffer to be flushed so that data is safely
c     written to disk
c
      call flush(51)
      if (lmoh) call flush(iohf)
      if (lmoh .and. ifohf.gt.0) call flush(ifohf)
      if (ifinst .gt. 0) call flush(ifinst)
c
      write (*,*) 'end of prtint'
      return
      end
