c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine rate_coeff(temp,press,ah2o,press1)
c
c	Updated by GZW on 15 January, 2001 to include PNA and other new rxn's.
c     
c       revised rxn #'s to match altered order in soa version of chem
c       (tcm, 10/01)
c
c In this subroutine, the rate constant at 298K is given in cm3 molecule-1 s-1 
c (see chread.f subroutine where this transition was made). 
c All data in this subroutine is taken from react_update.dat.
c
c Final output of this program will provide rate coefficients at grid 
c temperature and pressure in ppm-1 min-1 which will be used in step4 and its 
c called subroutines.
c     
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      include 'comtdr.cmd'
c     logical lprint
      dimension krset(17)
c
c  all the reactions in krset should be set in this routine
c
      data nset /17/
      data krset /2,5,6,10,19,24,27,31,32,33,36,41,42,45,52,53,65/

c     lprint = .false.
c     if ( abs(temp-298.) .lt. .01 .or. abs(temp-305.) .lt. .01) 
c    &     lprint = .true.
c
c air density (molecules/cm3) for each temp and press
c
c      if (lprint) then
c        write (*,*) temp,press,ah2o,press1
c      endif
c     
      airden = ((press/press1))*(1.0/(0.08206*temp))*(1.0e-3)*(6.02e+23)
c convert ah2o (given in ppm) into molecules cm-3
      ah2o = ah2o*(1.0e-6)*airden
c
c conversion factor to change cm3 molecule-1 s-1 into ppm-1 min-1
c
      ppmfac = airden*60.0*(1.0e-6)
cgzw      ppmfac1 = airden*60.0			!not needed?
c      do i = 1, nreact
c      write (*,*) i, rkn(i)
c      end do
c      write (*,*)
c      write (*,*) ppmfac, ppmfac1, airden, ah2o
c      write (*,*)
c
c Read rates for all three-body reactions
c
c o+no2+m
      a0 = 9.0e-32
      anr = 2.0
      ai = 2.2e-11
      amr = 0.0
      fc = 0.6
      call rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
      rk(5) = rate*ppmfac                 ! unit is ppm-1 min-1
c o+no+m
      a0 = 9.0e-32
      anr = 1.5
      ai = 3.0e-11
      amr = 0.0
      fc = 0.6
      call rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
      rk(6) = rate*ppmfac                 ! unit is ppm-1 min-1
c no3+no2+m                               ! Updated to NASA 2000 
      a0 = 2.0e-30
      anr = 4.4
      ai = 1.4e-12
      amr = 0.7
      fc = 0.6
      call rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
      rk(19) = rate                       ! unit is cm3 molecule-1 sec-1
c oh+no+m
      a0 = 7.0e-31
      anr = 2.6
      ai = 3.6e-11
      amr = 0.1
      fc = 0.6
      call rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
      rk(27) = rate*ppmfac                ! unit is ppm-1 min-1
c oh+no2+ m                               ! Updated to NASA 2000 
      a0 = 2.4e-30
      anr = 3.1
      ai = 1.7e-11
      amr = 2.1
      fc = 0.6
      call rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
      rk(31) = rate*ppmfac                ! unit is ppm-1 min-1
c ho2+no2+m
      a0 = 1.8e-31
      anr = 3.2
      ai = 4.7e-12
      amr = 1.4
      fc = 0.6
      call rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
      rk(32) = rate                       ! unit is cm3 molecule-1 sec-1
c c2o3+no2+m
      a0 = 9.7e-29
      anr = 5.6
      ai = 9.3e-12
      amr = 1.5
      fc = 0.6
      call rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
      rk(52) = rate                       ! unit is cm3 molecule-1 sec-1
c oh+so2+m
      a0 = 3.0e-31
      anr = 3.3
      ai = 1.5e-12
      amr = 0.0
      fc = 0.6
      call rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
      rk(65) = rate*ppmfac                ! unit is ppm-1 min-1
c
c read equilibrium constant data and calculate the rates for thermal rxns
c
c n2o5 <---> no3+no2                              ! Updated to NASA 2000 
      a0 = 3.0e-27
      anr = 10991.0
      eq24_19 = a0*exp(anr/temp)
      rk(24) = rk(19)*60.0/eq24_19            ! unit is min-1
      rk(19) = rk(19)*ppmfac               ! unit is ppm-1 min-1
c pna <---> ho2+no2
      a0 = 2.1e-27
      anr = 10900.0
      eq33_32 = a0*exp(anr/temp)
      rk(33) = rk(32)*60.0/eq33_32            ! unit is min-1
      rk(32) = rk(32)*ppmfac               ! unit is ppm-1 min-1
c pan  <---> c2o3+no2
      a0 = 9.0e-29
      anr = 14000.0
      eq53_52 = a0*exp(anr/temp)
      rk(53) = rk(52)*60.0/eq53_52            ! unit is min-1
      rk(52) = rk(52)*ppmfac               ! unit is ppm-1 min-1
c
c read data and calculate rates for special bimolecular rxns 35,41,40, and 44
c
c rxn 36: oh + hno3                         !  Updated to NASA 1997
      a0 = 7.2e-15
      anr = 785.0
      a1 = a0*exp(anr/temp)
      a0 = 4.1e-16
      anr = 1440.0
      a2 = a0*exp(anr/temp)
      a0 = 1.9e-33
      anr = 725.0
      a3 = a0*exp(anr/temp)
      rk(36) = (a1+(a3*airden)/(1+(a3*airden/a2)))*ppmfac !unit is ppm-1 min-1
c rxn 42: ho2+ho2+h2o
      a0 = 1.7e-33
      anr = -1000.0
      a1 = a0*exp(-anr/temp)
      a0 = 1.4e-21
      anr = 2200.0
      a2 = a0*exp(anr/temp)
      rk(42)=a1*airden*(1.0+ah2o*a2)*ppmfac              ! unit is ppm-1 min-1
c rxn 41: ho2+ho2
      a0 = 2.3e-13
      anr = -600.0
      a1 = a0*exp(-anr/temp)
      a0 = 1.4e-21
      anr = 2200.0
      a2 = a0*exp(anr/temp)
      rk(41)=a1*(1.0+ah2o*a2)*ppmfac                     ! unit is ppm-1 min-1
c rxn 45: co + oh
      a0 = 1.5e-13
      anr = 0.0
      a1 = a0*exp(-anr/temp)
      rk(45) = a1*(1+0.6*press/press1)*ppmfac   ! unit is ppm-1 min-1
c
c read data and calculate rate for rxn o + o2 + m
c
      a0 = 6.0e-34
      anr = 2.4
      rk(2) = a0*airden*(temp/300)**(-anr)
      rk(2) = rk(2)*0.20946*airden*60.0              ! here unit is min-1
c
c read data and calculate rate for rxn o1d + m
c
      a0 = 3.2e-11
      anr = -70.0
      a1 = a0*exp(-anr/temp)*0.20946*airden
      a0 = 1.8e-11
      anr = -110.0
      a2 = a0*exp(-anr/temp)*(1-0.20946)*airden
      rk(10) = (a1+a2)*60.0                          ! here unit is min-1
c
c
c TREATMENT FOR ALL OTHER REACTIONS
c     
c temperature factor for rate coefficients
c
      ttrate = (1.0/298) - (1.0/temp)
c
c change rate constant at 298K given in ppm-1 min-1 to cm3 molecule-1 s-1
c Divide rkn values by 1.478e+15 (as all values are given for surface level)
c and then multiply by ppmfac to change cm3 molecule-1 s-1 into ppm-1 min-1
c
c calculate rate constants for non-photoreactions
c
      do 100 i = 1, nreact
        if (.NOT. lphot(i)) then
           do n=1,nset
             if (i .eq. krset(n)) go to 100
           enddo
           rk (i) = rkn(i)*exp(acten(i)*ttrate)*ppmfac/(1.478e+15)
        end if
c       if (lprint) then 
c         write (*,*) i, rk(i)
c       endif
 100  continue
c
c   zero pna rates in warm conditions
c      if (temp.gt.293.15) then
c        rk(31) = 0.
c        rk(32) = 0.
c        rk(33) = 0.
c        rk(34) = 0.
c      endif
c      stop
c     
      return
c     
      end
c      
c Rate coefficients for termolecular rxns
c
      subroutine rate_cal(a0, anr, ai, amr, fc, temp, airden, rate)
          ak0 = a0*(temp/300)**(-anr)
          aki = ai*(temp/300)**(-amr)
          t1 = ak0*airden
          t2 = t1/aki
          t3 = t1/(1+t2)
          t4 = (alog10(t2))**2.0
          t5 = fc**(1.0/(1+t4))
          rate = t3*t5
          return
      end
      
