c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine rdfo3oh(iunit,nxf,nyf,nzf,ohcf,no3cf,ho2cf,h2o2cf,o3cf,
     &                   aso2cf,peccf)
c
c     Reads instantaneous O3 OH NO3 H2O2 HO2 fine grid concs from binary file
c
c----99/04/15: dfg
c    -modified from fgread for reading o3 and oh when lo3 is true.
c
c     Input arguments:
c               iunit:          fine grid instantaneous file unit number
c               nxf:            number of fine grid cells in x-direction
c               nyf:            number of fine grid cells in y-direction
c               nzf:            number of fine grid layers 
c     Output arguments:
c               ohcf:           fine grid concentrations (ppm)
c               o3cf:           fine grid concentrations (ppm)
c              NO3cf:           fine grid concentrations (ppm)
c             H2O2cf:           fine grid concentrations (ppm)
c              HO2cf:           fine grid concentrations (ppm)
c             aso2cf:           fine grid concentrations (ppm, in cld)
c              peccf:           fine grid concentrations (ugm/mole air)
c     Files read:
c               iunit:          fine grid instantaneous file
c     Files written:
c               6:              standard output file unit (*)
c
c  Called by update
c
      real*4 no3cf
      dimension o3cf(nxf,nyf,nzf),ohcf(nxf,nyf,nzf)
      dimension no3cf(nxf,nyf,nzf),h2o2cf(nxf,nyf,nzf),ho2cf(nxf,nyf,nzf)
      dimension aso2cf(nxf,nyf,nzf),peccf(nxf,nyf,nzf)
c
c  Read OH conc.
c
      do k=1,nzf
        read(iunit) ((ohcf(i,j,k),i=1,nxf),j=1,nyf)
      end do
c
c  Read HO2 conc.
c
      do k=1,nzf
        read(iunit) ((ho2cf(i,j,k),i=1,nxf),j=1,nyf)
      end do
c
c  Read NO3 conc.
c
      do k=1,nzf
        read(iunit) ((no3cf(i,j,k),i=1,nxf),j=1,nyf)
      end do
c
c  Read aq. SO2 conc.
c
      do k=1,nzf
        read(iunit) ((aso2cf(i,j,k),i=1,nxf),j=1,nyf)
      end do
c
c  Read H2O2 conc.
c
      do k=1,nzf
        read(iunit) ((h2o2cf(i,j,k),i=1,nxf),j=1,nyf)
      end do
c
c  Read ozone conc.
c
      do k=1,nzf
        read(iunit) ((o3cf(i,j,k),i=1,nxf),j=1,nyf)
      end do
c
c  Read pec conc.
c
      do k=1,nzf
        read(iunit) ((peccf(i,j,k),i=1,nxf),j=1,nyf)
      end do

      return
      end
