      subroutine step4 (dt, con, tcll, llpt, rcon,time,idate)
c     subroutine step4 (dt, con, tcll, llpt, rcon, iix, jjy, kkz, time)
c
c
c
c     Copyright 1996, 2001, 2002  Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
c
c  revised convergence criteria for fast species (tcm, 8/14/02)
c
c  new routine for step4 to incorporate the automatically generated 
c  chem routine.
c  Dongfen Gao -- Fast Solver
c
c--input variables
c         dt        r  time step (min)
c         con       r  concentrations (ppm)
c        rcon       r  radical concentrations (ppm)
c
c--output variables
c         con       r  new concentrations (ppm)
c        rcon       r  new radical concentrations (ppm)
c
c--local variables
c         hold      r  temporary conc vector
c        rhold      r  temporary radical conc vector
c         rate
c
c         iconv     i  =1 means failure to converge
c
c--routines called
c         chem
c         chem2a1
c         chem2a2
c         chem2b 
c
c
      include  'parchm.cmd'
      include  'comchem.cmd'
      include  'com3.cmd'

      dimension  con(nors), hold(nors),ctmp(nors)
      dimension  rcon(mxrads),rhold(mxrads)
      dimension  rate(nors), rp(nors,nors) 
      logical llpt
      common /idcell/icell,jcell,kcell,tmchm,ifg
c
      dimension rcon_save(mxrads), con_save(nors)
c
c     *** check dimensions
c
c     *** initialize iteration counters
c                    radicals-only flag
c
      data dchmin /1.0e-7/
c
      nspec = nors
c
c  save all inputs
c
c     if (llpt) then
c       write(*,*) 'nspec, CON: '
c       write(*,*) nspec, (con(l),l=1,nspec)
c     endif
      dt_save = dt
      do l=1,nspec
        con_save(l) =  con(l)
      enddo
      do l=1,mxrads
        rcon_save(l) = rcon(l)
      enddo
c
   15 itmax = 0
      lss  = .true.
c
c     *** set up time step parameters
c
      nitmax=10
      dttot = 0.
      dtmax=6.     ! minutes
      dtuse = amin1(dt,dtmax)
      hdt = 0.5*dtuse
      tdt = 0.5*dtuse
c
c     if (llpt) then
c       write(*,*) 'rk= ', (rk(i),i=1,mxgrx)
c     endif
c
c     if (ldark) then
c       call titrat(con(kno),con(kno2),rcon(kno3),con(ko3),tcll)
c     endif
c
  50  continue
c     if (llpt) then
c       write(*,*) 'itcnt,dtuse,dttot= ',itcnt,dtuse,dttot
c     endif
c
c--- save the total nitrogrn for balance check
c
      tnoy = con(kno)+con(kno2)+con(kpan)+con(khno3)+con(khno2)
     &     + con(kntr) + con(kpna)
c     if (llpt) write(*,*) 'tnoy = ', tnoy
c
      do 100 l=1,nspec
        hold(l) = con(l)
  100 continue
c
      do 110 l=1,mxrads
        rhold(l) = amax1(rcon(l),1.e-15)
  110 continue
c

      iter=0
  115 iter=iter+1
      iconv=0
c
      do l=1,nspec
        ctmp(l)=(hold(l)+con(l))/2.
      end do
c
 1333 format (7(1x,e10.4))
c
c***
      if (ldark) then
c
c      nighttime, assume no,o3 and no3 react immediately
c      O and O1D are zero.
c
        rhold(ko)=1.0E-30
        rhold(ko1d)=1.0E-30
c
        if (hold(kno).lt.1.e-4.and.hold(ko3).lt.1.e-4) then
c          call chem2b (nspec,ctmp,rhold,rate,rp,lerror)
        elseif (hold(ko3).lt.1.0e-6) then
c
c  When chem2a1 is called, this means that NO has won the titrations.
c  O3 concentration is set to zero.  So does for NO3 and N2O5.
c
c
           rhold(kno3)=1.0e-19
           rhold(kn2o5)=1.0e-19
           ctmp(ko3)=1.0e-12
c          call chem2a1 (nspec,ctmp,rhold,rate,rp,lerror)
        elseif (hold(kno).lt.1.0e-6) then
c
c  When chem2a2 is called, this means that O3 has won the titrations.
c  NO concentration is set to zero.
c
           ctmp(kno)=1.0e-12
c          call chem2a2 (nspec,ctmp,rhold,rate,rp,lerror)
        endif
      else
c
c  Call the daytime chemistry solver
c
        call chem (nspec,ctmp,rhold,rate,rp,lerror)
      endif
      ispend = 4
      if (lss) ispend = 3
c     if (llpt) then
c       write(*,*) 'After chem, ispend, lss, rhold: ', ispend,lss
c       write(*,*) rhold
c     endif
c
      if (lerr)  then
        write(*,*) 'error in chem -- return true for lerr'
        write(*,*) 'ier,iter,nitmax= ',ier,iter,nitmax
        write(*,1200) (n,rate(n),n=1,nors)
        write(*,1300) (n,hold(n),n=1,nors)
        write(*,1400) (n,rk(n),n=1,64)
        call flush (6)
        stop
      endif
c
      do 130 i=1,ispend
        do 120 j=1,ispend
          rp(i,j)=-0.5*hdt*rp(i,j)
  120   continue
  130 continue
c
        do 140 i=1,ispend
          rp(i,i)=1.+rp(i,i)
  140   continue
c
        do 150 l=1,ispend
          rate(l) = -(hold(l) - con(l) - hdt*rate(l))
  150   continue
c       if (llpt) then
c         write(*,*) 'rate: ', rate
c         write(*,*) 'hold: ',hold
c         write(*,*) 'con: ',con
c       endif
c
        if (ldark) then
          if (hold(kno).lt.1.e-4.and.hold(ko3).lt.1.e-4) then
            call colpivot(1,3,nspec,rp,rate,ier)
            ls1=1
            ls2=3
          elseif (hold(kno).lt.1.0e-6) then
            call colpivot(2,3,nspec,rp,rate,ier)
            ls1=2
            ls2=3
            rate(kno)=0.
          elseif(hold(ko3).lt.1.0e-6) then
            call colpivot(1,2,nspec,rp,rate,ier)
            ls1=1
            ls2=2
            rate(ko3)=0.
          endif
        else
          call colpivot(1,ispend,nspec,rp,rate,ier)
          ls1=1
          ls2=ispend
        endif
c
        if (ier.eq.0) then
          lerr=.true.
          write(*,*) 'error in concentration calculation'
          write(*,*) 'in colpivot, matrix of equations is singular'
          write(*,*) 'ier,iter,nitmax= ',ier,iter,nitmax
          write(*,*) 'lss, ispend: ', lss, ispend
          write(*,*) 'icell,jcell,kcell,tmchm,ifg: '
          write(*,*) icell,jcell,kcell,tmchm,ifg
          write(*,1200) (n,rate(n),n=1,nors)
          write(*,1300) (n,hold(n),n=1,nors)
          write(*,1400) (n,rk(n),n=1,64)
          call flush (6)
          stop
        endif
c
        tmpmax=0.
c        do 160 l = 1,4
        do 160 l = ls1, ls2
          tmphh=hold(l)
c         if (llpt) then
c           write(*,*) 'l, hold(l), rate(l), tmphh: '
c           write(*,*) l, hold(l), rate(l), tmphh
c         endif
          if (hold(l).gt.1.0e-9) then
             err=abs(rate(l)/hold(l))
          else
             err=abs(rate(l))
          endif
          hold(l)=tmphh + rate(l)
          if (err.gt.tmpmax) then
            tmpmax=err
          endif
c         if (hold(l).le. 0.0) hold(l)=tmphh/10.
          if (hold(l).le. -bdnl(l)) tmpmax = 10.*rerror
  160   continue
c       if (llpt) then
c         write(*,*) 'lss = ', lss
c         write(*,*) 'hold(kno2),hold(kno2s),rhold(kno3),rhold(kn2o5):'
c         write(*,*) hold(kno2),hold(kno2s),rhold(kno3),rhold(kn2o5)
c       endif
        if (lss) hold(kno2) = hold(kno2s)+rhold(kno3)+2.*rhold(kn2o5)
c
        if (tmpmax.le. rerror) then
c
c  now values converged, save concentrations, and check if we finished
c  the original step.
c
          if (itmax.lt.iter) itmax = iter
c         do 200 l=ls1,ls2
          do 200 l=1,4
            if (hold(l).lt.bdnl(l)) hold(l)=bdnl(l)
            con(l) = hold(l)
  200     continue
          do 210 l=1,mxrads
            rcon(l) = rhold(l)
  210     continue
c
c   Fast species converged, now calculate slow species 
c   by explicit method
c
          if (ldark) then
            if (hold(kno).lt.1.e-4.and.hold(ko3).lt.1.e-4) then
c             call slowntb(nspec,con,rcon,rate)
            elseif (hold(kno).lt.1.0e-6) then
c             call slownta2(nspec,con,rcon,rate)
            elseif (hold(ko3).lt.1.0e-6) then
c             call slownta1(nspec,con,rcon,rate)
            endif
          else
            call slowdr(nspec,con,rcon,rate)
          endif
c
c         call diagjac (nspec,con,rcon,rp)
c
c       if (llpt) then
c         write(*,*) 'Before 250'
c         write(*,*) 'rate: ', rate
c         write(*,*) 'hold: ',hold
c         write(*,*) 'con: ',con
c         write(*,*) 'kNO, kNO2S, KHNO2, H2O: '
c         write(*,*) kNO, kNO2S, KHNO2, H2O
c         write(*,*) 'rk(28),rk(29),OH,rk(30),HNO2: '
c         write(*,*) rk(28),rk(29),OH,rk(30),HNO2
c         write(*,*) 'rk(26),NO,NO2S,rk(27): '
c         write(*,*) rk(26),NO,NO2S,rk(27)
c         write(*,*) 'Rate(kHNO2): ', Rate(kHNO2)
c       endif
          do 250 l=5,nors
c           xloss=rp(l,l)
c           xprod=rate(l)-xloss*con(l)
c           if (xloss.lt.1.0e-6) then
c             con(l)=con(l) + hdt*xprod
c           else
c             tmp=xloss*con(l) + xprod
c             con(l)=(1./xloss)*(-xprod + tmp)*exp(xloss*hdt)
c           endif 
            con(l) = con(l) + hdt*rate(l)
            if (l .eq. kpna) cpna = con(kpna)
c           if (llpt) then
c             write(*,*) 'l, con(l), bdnl(l): ', l, con(l), bdnl(l)
c           endif
            if (con(l).lt.bdnl(l)) then
              con(l)=bdnl(l)
           end if
  250     continue
c        if (llpt) then
c          write(*,*) 'After 250'
c          write(*,*) 'rate: ', rate
c          write(*,*) 'hold: ',hold
c          write(*,*) 'con: ',con
c        endif
c
c  if pna < 0, then reset everything, calculate pna in ss, set pna
c  rates to zero, and try again.
c
        if (cpna .lt. -bdnl(kpna)) then
          dt = dt_save
          do l=1,nspec
            con(l) =  con_save(l)
          enddo
          do l=1,mxrads
            rcon(l) = rcon_save(l)
          enddo
          pna = rk(32)*rcon(kHO2)*con(kNO2S) 
     &          / (rk(33)+rk(34)*rcon(kOH)+rk(35))
          con(kno2) = con(kno2) + amax1(0.,con(kpna)-pna)
          con(kpna) = amax1(bdnl(kpna),pna)
          rk(32) = 0.
          rk(33) = 0.
          rk(34) = 0.
          rk(35) = 0.
          go to 15
        endif
c       if (llpt) then
c         write(*,*) 'Before calc tnoy1'
c         write(*,*) 'rate: ', rate
c         write(*,*) 'hold: ',hold
c         write(*,*) 'con: ',con
c       endif
c
c--- calculate total nitrogen after concentration converged.
c    if total nitrogen is changes above 1%, we will force
c    the total nitrogen to be balance.
c
          tnoy1 = con(kno)+con(kno2)+con(kpan)+con(khno3)+con(khno2)
     &          + con(kntr) + con(kpna)
c         if (llpt) write(*,*) 'tnoy1 = ', tnoy1
c         difnoy=(tnoy1-tnoy)/tnoy
c         if ( abs(difnoy) .gt. 0.01) then
c           write(*,*) 'nitrogen balance not maintained.'
c           write(*,*) difnoy
c           if (tnoy.gt.1.0e-4) then
c            con(kno)=con(kno)*tnoy/tnoy1
c            con(kno2s)=con(kno2s)*tnoy/tnoy1
c            con(kno2)=con(kno2)*tnoy/tnoy1
c            con(khno3)=con(khno3)*tnoy/tnoy1
c            con(khno2)=con(khno2)*tnoy/tnoy1
c            con(kpna)=con(kpna)*tnoy/tnoy1
c            con(kpan)=con(kpan)*tnoy/tnoy1
c            con(kntr)=con(kntr)*tnoy/tnoy1
c           endif
c         endif

          dttot = dttot + hdt 
          if (dttot .ge. dt*0.9999) go to 9999
c
c  for next time step, use a little big value than the current values,
c      but less than the original time step.
c
          if (1.5*hdt .lt. tdt) then
            hdt=1.5*hdt 
          endif
          if (dttot + hdt .gt. dt) then
            hdt = dt-dttot
          endif
          go to 50
c
        else
c
c  fail to converge, next iteration ?
c
          if (iter .le. nitmax) then
            iconv=1
            go to 115
          else
c
c  still fail to converge after nitmax times, time step must be cut
c
            hdt=hdt/2.0

            if (hdt .lt. dtmin4) then
              write(*,*) 'time step is smaller than input from',
     &                    'the user: hdt, dtmin= ', hdt,dtmin4
              write (*,2901) hdt
c
              write (*,*) 'i, j, k, ifg: ', icell, jcell, kcell,ifg
              write (*,2910) (l, con(l), hold(l), l=1,nspec)
              write (*,2950) (l, rcon(l), rhold(l), l=1,mxrads)
              write (*,2920) (l, rate(l),l=1,nors)
              write (*,2930)
              do 500 il=1,nors
                write (*,2940) il, (rp(il,jl), jl=1,nors)
  500         continue
              call flush (6)
              lerr=.true.
              stop
            endif

            if (hdt .lt. dchmin) then
              write(*,*) 'time step is samller than dchmin: ',
     &                    'hdt, dchmin= ', hdt,dchmin
              write (*,2901) hdt
              lerr=.true.
              stop
            endif
            go to 50
          endif
        endif
c
c---------------------------------------------------------------
 1200 format(1x,'   rate  ',/,(1x,i4,e12.5))
 1300 format(1x,'   hold  ',/,(1x,i4,e12.5))
 1400 format(1x,'     rk  ',/,(1x,i4,e12.5))
 2900 format (23h0error in -step4- cell , 3(i4, 2h, ), 10hspecies = ,
     &  i2, 10h  segment=, i2)
 2901 format (34h could not reduce step size below , f8.3)
 2910 format (5h0spec, 9x,3hcon, 8x,4hhold / (i5, 1p2e12.3))
 2920 format (5h0spec, 8x,4hrate, 10x,2hdc / (i5, 1pe12.3))
 2930 format (/// 10h rate(i,j) / 3x, 12i10)
 2940 format ((i5, (1p12e10.2)))
 2950 format (5h0spec, 9x,4hrcon, 8x,5hrhold / (i5, 1p2e12.3))
c---------------------------------------------------------------------
c
 9999 continue
 101    format (3(1x,i3),100(1x,e10.4))
c
      return
      end
