c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine stepx_04(deltat,time,ndate)
c
c-----ATDM v1.00
c
c     Performs horizontal advection in x direction for coarse grid
c
c-----95/02/22:
c     -from UAM-V STEPX.F and STEPY.F Revision 1.2  94/02/10  21:01:08  tcm
c     -removed call to TERMS
c     -moved call to WETDEP from TERMS to this routine
c     -removed DECAY array
c     -removed d() array from common SHAS
c     -revised calculation of DPFLOW to include all layers, not just layer 1,
c      using RDECAY straight out of WETDEP routine
c     -modified SMOLAR concentration vector CX() to include reduction due to 
c      wet deposition, using RDECAY straight out of WETDEP
c-----95/06/09:
c     -removed all code related to wet deposition (done in cloud sub-model)
c     -removed all code related to vertical advection (Lorenz cycle)
c     -advection performed for the quantity C* = P* x C
c-----95/09/18:
c     -restructured to scale C by P* value at beginning of time step, then
c      return to C after advection by dividing by P* value at end of time step
c     -alternating x/y advection for odd/even time step index
c     -added latitudinal cell scaling for y-advection to account for
c      cell width as a function of lat
c-----95/09/19:
c     -moved call to HGTNEW from ATDM to here to obtain new P* values
c-----96/12/05:
c     -moved call to HGTNEW back to ATDM ... conc( ) is now coming in as
c      conc()*Pstar .... will be divided by P* again in ATDM
c
c     Input arguments:
c               deltat:         time step (min)       
c               time:           model time
c               ndate:          model date
c     Files written:
c               6:              standard output file unit (*)
c     Routines called:
c               smolar    hgtnew
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'ngmap.cmd'
      include 'com3.cmd'
      include 'dtksub.cmd'
c
      common /shas/ w(mxxx),cx(mxxx),cb(mxxx),
     &              jadf(mxxx),nm1,b1,b2
      data maxpas /15/
c
      write(*,3001) deltat
 3001 format(' enter stepx_04  ',f10.2)
c      write(*,*) 'deltat,time,ndate '
c      write(*,*) deltat,time,ndate
      do 40 k = 1,nlayer
      timlftk=deltat
       do 26 nstp=1,maxpas
c	 write(*,*) ' STEPX_04 ... nstp, timlft :',nstp,timlftk
	 if ( timlftk .le. 0.0001 ) goto 40
c
	 if ( deltatk(k) .le. timlftk ) then
	    dtk=deltatk(k)
         else
	    dtk=timlftk
	 endif
c
c-----------------------------------------------------------------------
c  X ADVECTION
c
c  integrate one species at a time, one row at a time.
c
  10  do 100 j = 1,noy
c
       timlftj=dtk
c
c     check for out side of modeling region
c
        if (istr(j).ge.nox) goto 100
        i1 = istr(j)
        i2 = iend(j)
        nm1 = i2 - i1
c
        do 95 nstpj=1,maxpas
	 if ( timlftj .le. 0.0001 ) goto 100
c
	 if ( dtsi(j,k) .le. timlftj ) then
	    dtj=dtsi(j,k)
         else
	    dtj=timlftj
	 endif
c
c     for all species
c
        do 150 l = 1,nospec
c
c     call advection scheme for each layer
c
            do 20 i = i1,i2
              ii = i - i1 + 1
	      if ( nstpj .eq. 1 .and. nstp .eq. 1 ) then
                cx(ii) = conc(i,j,l,k)
                cb(ii) = kh(i,j,k)*dtj/deltax(j)**2
                w(ii) = u(i,j,k)
                if (mod(ingmap(i,j),10) .eq. 0) then
                  jadf(ii) = 0
                else
                  jadf(ii) = 1
                endif
	      else
                cx(ii) = conc(i,j,l,k)
                cb(ii) = kh(i,j,k)*dtj/deltax(j)**2
	      endif
   20       continue
            b1 = bc(j,4,l,k)
            b2 = bc(j,3,l,k)
c
            call smolar(dtj,deltax(j))
c
            do 25 i = i1,i2
              ii = i - i1 + 1
c              conc(i,j,l,k) = cx(ii)
              conc(i,j,l,k) = amax1(bdnl(l),cx(ii))	      
   25       continue
c
  150   continue
c
	timlftj=timlftj-dtj
c
  95    continue
c
  100 continue
c
	  timlftk=timlftk-dtk
c
   26      continue
   40     continue
c
      return
      end
