c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine stepzf(nxf,nyf,nzf,nspecf,dtf,sigdotf,htf,
     &                  vgf,cncf,idebug,ifine)
c
c-----ATDM v1.00
c
c     Computes vertical advection using Botts scheme for non-uniform grids
c     in sigma coordinates
c     FINE GRID VERSION
c
c-----95/06/13:
c     C Emery SAI
c-----95/06/22:
c     -added idebug to argument list
c-----95/06/26:
c     -added sedimentation velocity component calculated in PARTSED
c      to sigdotf
c     -added htf and vgf to argument list
c-----95/07/14:
c     -removed calculation of sub-steps as this is an implicit scheme
c-----95/10/04:
c     -restructured code that adds vg to sigdot
c-----95/11/08:
c     -added code to skip cells covered by a fine grid
c-----95/12/12:
c     -added code to treat vgf for nh4s as average of gso4 and aso4
c
c     Based on routines from SAQM/RADM
c
c     Input arguments:
c               nxf:            number of fine grid cell in x-direction
c               nyf:            number of fine grid cell in y-direction
c               nzf:            number of fine grid layers
c               nspecf:         number of modeled species
c               dtf:            fine grid time step (min)
c               sigdotf:        array of fine grid vertical velocities (1/s)
c               htf:            array of fine grid layer intrface heights (m)
c               vgf:            array of fine grid sedimentation vels (m/s)
c               cncf:           array of fine grid concentrations (ppm)
c               idebug:         diagnostic debug flag
c               ifine:          fine grid index
c     Output arguments:
c               cncf:           array of fine grid concentrations (ppm)
c     Called by:
c               atdm 
c
      include 'parm.cmd'
      include 'parfin.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'deffin.cmd'
c
      dimension ixff0(maxfin), ixff1(maxfin),
     &          jyff0(maxfin), jyff1(maxfin)
      dimension sigdotf(nxf,nyf,nzf),htf(nxf,nyf,nzf),
     &          vgf(nxf,nyf,nzf,nspecf),cncf(nxf,nyf,nspecf,nzf) 
      real i_minus(mxfz+1),i_plus(mxfz+1),wf(mxfz),vciflux(mxfz+1)
      real sigmah(mxfz),dsigh(mxfz),dtodz(mxfz),dsigi(mxfz),vci(mxfz)
      real a1co(3,mxfz),a2co(3,mxfz),vrtvel(mxfz)
      logical lprnt
c
c-----Bit 8 (128's location) turns on debug writes in stepz
c
      idiag = and(idebug,128)
      if (idiag.ne.0) write(*,*)'Entering STEPZF, deltf = ',dtf
c
c----Set up some parameters relating the grids
c
      do 5 if1 = 1,numfin
        if (ifgptr(if1) .ne. ifine) go to 5
        ixff0(if1) = (ixfb(if1) - ixfb(ifine))*nhf(ifine) + 2
        ixff1(if1) = (ixfe(if1) - ixfb(ifine) + 1)*nhf(ifine) + 1
        jyff0(if1) = (jyfb(if1) - jyfb(ifine))*nhf(ifine) + 2
        jyff1(if1) = (jyfe(if1) - jyfb(ifine) + 1)*nhf(ifine) + 1
        if (ixff0(if1).le.2 .or. ixff1(if1).ge.nxf .or.
     &      jyff0(if1).le.2 .or. jyff1(if1).ge.nyf) then
          write(*,*) ' erroneous grid parameters in stepzf'
          write(*,*) ' ixff0, ixff1, jyff0, jyff1: ',ixff0,ixff1,
     &                 jyff0, jyff1
          write(*,*) 'if1, ifine: ',if1,ifine
          stop
        endif
   5  continue
c
c-----Calculate depth parameters
c
      kmax = nzf
      do 10 k = 1,kmax
        sigmah(k) = (sigmaf(k,ifine) + sigmaf(k+1,ifine))/2.
        dsigh(k) = sigmaf(k+1,ifine) - sigmaf(k,ifine)
   10 continue
      do 11 k = 1,kmax-1
        dsigi(k) = 1./(sigmah(k+1) - sigmah(k))
   11 continue
c
c-----Compute polynomial coefficients
c
c     coefficients for k=1 and k=nlayer
c
c      a1co(1,1) = dsigh(1)*dsigi(1)/8.
c      a1co(2,1) = -a1co(1,1)
c      a1co(2,kmax) = dsigh(kmax)*dsigi(kmax-1)/8.
c      a1co(3,kmax) = -a1co(2,kmax)
c
c     for k = 2 to nlayer-1
c
c      do 20 k = 2,kmax-1
c        dod = dsigi(k)/dsigi(k-1)
c        dpd = sigmah(k+1) - sigmah(k-1)
c        ds2o24 = dsigh(k)*dsigh(k)/24.
c        a1co(1,k) = dod/dpd*dsigh(k)/8.
c        a1co(2,k) = (1. - dod)*dsigi(k-1)*dsigh(k)/8.
c        a1co(3,k) = -1./dod/dpd*dsigh(k)/8.
c        a2co(1,k) = dsigi(k)/dpd*ds2o24
c        a2co(2,k) = -dsigi(k-1)*dsigi(k)*ds2o24
c        a2co(3,k) = dsigi(k-1)/dpd*ds2o24
c 20   continue
c
c-----Loop over rows and columns
c
      do 100 j = 2,nyf-1
        do 120 i = 2,nxf-1 
c
c-----Skip cells covered by another finer grid
c
          do 130 if1 = 1,numfin
            if (ifgptr(if1).eq.ifine) then
              if (i.ge.ixff0(if1) .and. i.le.ixff1(if1)
     &            .and. j.ge.jyff0(if1) .and. j.le.jyff1(if1))
     &            go to 120
            endif
  130     continue
c
        lprnt = .false.
c        if (i.eq.17 .and. j.eq.11) lprnt = .true.
        if (i.eq.4 .and. j.eq.nyf/2) lprnt = .true.
c
c-----Loop over all species; load work array vci
c     and add particle sedimentation velocity to sigmadot
c
          do 150 l = 1,nspecf
            do 200 k = 1,kmax
              vci(k) = cncf(i,j,l,k)
c
              if (k.lt.kmax) then
c
c --- treat nh4s as weighted average of aso4 and gso4
c
	        if (l.eq.knh4s) then
		  vgtf=cncf(i,j,kgso4,k+1)+cncf(i,j,kaso4,k+1)
		  if ( abs(vgtf) .gt. 0.0 ) then
		    vgf(i,j,k+1,knh4s)=
     &                    (vgf(i,j,k+1,kgso4)*cncf(i,j,kgso4,k+1)+
     &                     vgf(i,j,k+1,kaso4)*cncf(i,j,kaso4,k+1))/vgtf
		  else
		    vgf(i,j,k+1,knh4s)=0.0
	   	  endif
		endif
c
                if (vgf(i,j,k+1,l).gt.0.) then
                  pstim = (htf(i,j,k+1) - htf(i,j,k))/vgf(i,j,k+1,l)
                  vrtvel(k) = sigdotf(i,j,k) + abs(dsigh(k+1))/pstim
                else
                  vrtvel(k) = sigdotf(i,j,k)
                endif
              else
                vrtvel(k) = 0.
              endif
 200        continue
c
c-----Determine number of vertical advection sub-steps
c
c           dtmin = 60.*dtf
c           do 20 k = 1,kmax-1
c             dt = amin1(abs(dsigh(k)),abs(dsigh(k+1)))/
c    &                  (2.*abs(vrtvel(k)))
c             dtmin = amin1(dtmin,dt)
c  20       continue
c           if (dtmin.ge.60.*dtf) then
c             ndt = 1
c           else
c             ndt = int(60.*dtf/dtmin) + 1
c           endif
            ndt = 1
            dt = 60.*dtf/float(ndt)
            do 22 k = 1,kmax
              dtodz(k) = dt/abs(dsigh(k))
   22       continue
c
c-----Loop over sub-steps; initialize vertical arrays
c
            do 160 n = 1,ndt
c
c            do 201 k = 1,kmax+1
c              i_minus(k) = 0.
c              i_plus(k)  = 0.
c 201        continue
cc
cc-----First order for layer 1 and kmax 
cc
c            a0 = vci(1)
c            a1 = a1co(1,1)*vci(2) + a1co(2,1)*vci(1)
c            if (vrtvel(1).lt.0.) then
c              cj_plus = -vrtvel(1)*dtodz(1)
c              x1 = 1. - 2.*cj_plus
c              x2 = x1*x1
c              temp = a0*cj_plus + a1*(1. - x2)
c              i_plus(2) = amax1(0.,temp)
c            endif
c            weight = amax1(i_plus(2),1.e-10,a0)
c            wf(1) = vci(1)*dsigh(1)/weight
cc
c            a0 = vci(kmax)
c            a1 = a1co(2,kmax)*vci(kmax) + a1co(3,kmax)*vci(kmax-1)
c            if (vrtvel(kmax-1).gt.0.) then
c              cj_minus = vrtvel(kmax-1)*dtodz(kmax)
c              x1 = 1. - 2.*cj_minus
c              x2 = x1*x1
c              temp = a0*cj_minus - a1*(1. - x2)
c              i_minus(kmax) = amax1(0.,temp)
c            endif
c            weight = amax1(i_minus(kmax),1.e-10,a0)
c            wf(kmax) = vci(kmax)*dsigh(kmax)/weight
cc
cc-----Second order scheme for k = 2 to kmax-1
cc
c            do 210 k = 2,kmax-1
cc
c              a0 = vci(k)
c              a1 = a1co(1,k)*vci(k+1) + a1co(2,k)*vci(k) +
c     & 	           a1co(3,k)*vci(k-1)
c              a2 = a2co(1,k)*vci(k+1) + a2co(2,k)*vci(k) +
c     &             a2co(3,k)*vci(k-1)
cc
c              if (vrtvel(k-1).gt.0.) then
c                cj_minus = vrtvel(k-1)*dtodz(k)
c                x1 = 1. - 2.*cj_minus
c                x2 = x1*x1
c                temp = a0*cj_minus - a1*(1. - x2) + a2*(1. - x1*x2)
c                i_minus(k) = amax1(0.,temp)
c              endif
cc
c              if (vrtvel(k).lt.0.) then
c                cj_plus = -vrtvel(k)*dtodz(k)
c                x1 = 1. - 2.*cj_plus
c                x2 = x1*x1
c                temp = a0*cj_plus + a1*(1. - x2) + a2*(1. - x1*x2)
c                i_plus(k+1) = amax1(0.,temp)
c	      endif
cc
c              weight = amax1(i_plus(k+1)+i_minus(k),1.e-10,a0+2.*a2)
c              wf(k) = vci(k)*dsigh(k)/weight
c
c  210       continue
c
c-----Apply layer fluxes to obtain new concentration profile
c
c
	    flux2=0.
            do 210 k = 1,kmax-1
	      flux1=flux2
	      kval=k
              if (vrtvel(k).gt.0.) kval=kval+1
 	      flux2=vci(kval)*vrtvel(k)
	      vci(k)=vci(k)+dtodz(k)*(flux2-flux1)
  210       continue
	    flux1=flux2
c	    flux2=vrtvel(kmax)*vci(kmax)
   	    flux2=0.0
c	    if(vrtvel(kmax) .gt. 0.) flux2=vrtvel(kmax)*ctop
	    vci(kmax)=vci(kmax)+dtodz(kmax)*(flux2-flux1)
c
c------End of sub-step loop; load final results back into conc
c
  160       continue
            do 170 k = 1,kmax
              if (lprnt .and. idiag.ne.0) 
     &          write(*,*)'l,k,conc,vci',l,k,cncf(i,j,l,k),vci(k)
              cncf(i,j,l,k) = amax1(vci(k),bdnl(l))
  170       continue
c
  150     continue
  120   continue
  100 continue
      return
      end
