c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine surfce(nm,gsun,ustrw,wsrf,tcellk,rhum,lstrcl,
     &                  lsnocl,flndcl,raincl,rs)
c
c-----ATDM v1.00
c
c     Surface layer resistance calculated as a function of season, stabilty 
c     and land use type
c     From M.L. Wesely, Atmos Environ 23(6):1293-1304 (1989)
c
c-----95/04/03:
c     -from UAM-V Revision 1.2  94/02/10  21:01:34  tcm
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c     -major modifications for ATDM species
c     -moved DRATIO and FREACT to AQBDAT
c     -moved calculation of RCL and RGS to near top of 270 loop so that
c      it is calculated each landuse iteration independent of water,
c      dew-wetted, or dry surface
c-----95/11/03:
c     -corrected calculation of rcutwet to properly check for NO and NO2,
c      and to access proper value of hheff and freact
c-----95/06/26:
c     -corrected calc of rcutwet to point to SO2 and O3 (ll=0), and to 
c      access proper value of freact and dratio, etc. ...
c
c     Gaseous depositing species are assumed to be soluble and/or reactive
c     note that the indeces here are not the same as for the concentrations
c
c     Input arguments:
c               nm:             month
c               gsun:           solar insolation (W/m2)
c               ustrw:          friction velocity over water (m/s)
c               wsrf:           10 m wind speed (m/s)
c               tcellk:         ambient temperature (K)
c               rhum:           ambient relative humidity (%)
c               lstrcl:         stress code
c               lsnocl:         snow code
c               flndcl:         array of fractional land use values
c               raincl:         rainfall rate (in/hr)
c     Output arguments:
c               rs:             array of surface resistances (s/m)
c     Called by:
c               surlyr    srlryf
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
c
      dimension rmin(mlus,5),rcuts(mlus,5),rcls(mlus,5),rgss(mlus,5),
     &          rac(mlus,5),rclo(mlus,5),rgso(mlus,5)
      dimension rsurf(mlus,0:mxgas),rstom(mlus,0:mxgas),rmes(0:mxgas),
     &          rcut(mlus,0:mxgas), rcl(mlus,0:mxgas),rgs(mlus,0:mxgas),
     &          rcutwet(mlus,0:mxgas),rstrf(mlus,0:mxgas)
      dimension rs(mxgas,mlus), wsrf(mlus)
      dimension tmax(mlus), tmin(mlus), topt(mlus)
      dimension flndcl(mlus)
      dimension hheff(0:mxgas)
      logical ldew
c
c  the following resistances are from wesely (1989)
c
      data rmin/
     & 99999,60.  ,120. ,70.  ,130,100,99999,99999,80.  ,100. ,150. ,
     & 99999,99999,99999,99999,250,500,99999,99999,99999,99999,99999,
     & 99999,99999,99999,99999,250,500,99999,99999,99999,99999,99999,
     & 99999,99999,99999,99999,400,800,99999,99999,99999,99999,99999,
     & 99999,120. ,240. ,140. ,250,190,99999,99999,160. ,200. ,300. /
c
      data rcuts/
     & 99999,2000.,2000.,2000.,2000,2000.,99999,99999,2500,2000,4000,
     & 99999,9000.,9000.,9000.,4000,8000.,99999,99999,9000,9000,9000,
     & 99999,99999,9000.,9000.,4000,8000.,99999,99999,9000,9000,9000,
     & 99999,99999,99999,99999,6000,9000.,99999,99999,9000,9000,9000,
     & 99999,4000.,4000.,4000.,2000,3000.,99999,99999,4000,4000,8000/
c
      data rac/
     & 100,200,100,2000,2000,2000,0.5,0.5,300,150,200,
     & 100,150,100,1500,2000,1700,0.5,0.5,200,120,140,
     & 100,10.,100,1000,2000,1500,0.5,0.5,100,50.,120,
     & 100,10.,10.,1000,2000,1500,0.5,0.5,50.,10.,50.,
     5 100,50.,80.,1200,2000,1500,0.5,0.5,200,60,120./
c
      data rgss/
     & 400,150,350,500,500,100,0.5,1000,0.5,220,400,
     & 400,200,350,500,500,100,0.5,1000,0.5,300,400,
     & 400,150,350,500,500,200,0.5,1000,0.5,200,400,
     & 100,100,100,100,100,100,0.5,1000,100,100,50.,
     & 500,150,350,500,500,200,0.5,1000,0.5,250,400/
c
      data rgso/
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200.,
     & 300,150.,200.,200.,200.,300.,2000,400,800.,180.,200.,
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200.,
     & 600,3500,3500,3500,3500,3500,2000,400,3500,3500,3500,
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200./
c
      data rcls/
     & 99999,2000.,2000.,2000,2000,2000,99999,99999,2500,2000.,4000,
     & 99999,9000.,9000.,9000,2000,4000,99999,99999,9000,9000.,9000,
     & 99999,99999,9000.,9000,3000,6000,99999,99999,9000,9000.,9000,
     & 99999,99999,99999,9000,200.,400.,99999,99999,9000,99999,9000,
     & 99999,4000.,4000.,4000,2000,3000,99999,99999,4000,4000.,8000/
c
      data rclo/
     & 99999,1000,1000,1000,1000,1000,99999,99999,1000,1000,1000,
     & 99999,400.,400.,400.,1000,600.,99999,99999,400.,400.,400.,
     & 99999,1000,400.,400.,1000,600.,99999,99999,800.,600.,600.,
     & 99999,1000,1000,400.,1500,600.,99999,99999,800.,1000,800.,
     & 99999,1000,500.,500.,1500,700.,99999,99999,600.,800.,800./
c
      data tmax /40.,40.,40.,40.,40.,40.,40.,40.,40.,40.,40./
      data tmin / 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0./
      data topt /20.,20.,20.,20.,20.,20.,20.,20.,20.,20.,20./
c
c    ** need to determine iseasn from month and snow cover **
c       iseasn = 1  summer
c                2  early autumn
c                3  late autumn after frost
c                4  winter with snow on ground
c                5  spring 
c 
      if (ngas .eq. 0) then
c        write (*,*) 'going at the end of subroutine as ngas is equal to', ngas
         go to 315
      end if
      if (nm .ge. 11) then 
        iseasn = 3
      elseif (nm .eq. 10) then
        iseasn = 2
      elseif (nm .ge. 6) then
        iseasn = 1
      elseif (nm .ge. 3) then
        iseasn = 5
      else
        iseasn = 3
      endif
      if(lsnocl .gt. 0) iseasn = 4
c 
c   calculate effective Henry's Law coefficients f(T,pH)
c   assume leaf surface pH = 7 (recommended by Wesely)
c
       hplus = 1.e-7
       hplus2 = hplus*hplus
       tfac = 1./298. - 1./tcellk
       do 100 l = 0,ngas
	  if ( l .eq. 0 ) then
	    ll=0
	  else
            ll = jind(l)
	  endif
          ekh  = ekh0(ll)*exp(tdep(ll)*tfac)
          ek1d = ek1d0(ll)*exp(tdep1d(ll)*tfac)
          ek2d = ek2d0(ll)*exp(tdep2d(ll)*tfac)
          hheff(ll) = ekh + ekh*ek1d/hplus + ekh*ek1d*ek2d/hplus2
  100 continue
c     if (lprt) then
c       write(*,*) 'In surfce, hheff(khg0(4)), hheff(khg2(4)): '
c       write(*,*) hheff(khg0(4)), hheff(khg2(4))
c     endif
      if (kso2 .gt. 0) then
        jso2 = jind(kso2)
        heffso = hheff(jso2)
      else
        ekh  = ekh0(14)*exp(tdep(14)*tfac)
        ek1d = ek1d0(14)*exp(tdep1d(14)*tfac)
        ek2d = ek2d0(14)*exp(tdep2d(14)*tfac)
        heffso = ekh + ekh*ek1d/hplus + ekh*ek1d*ek2d/hplus2
      endif
c     write(*,*) ' HEFFSO :',heffso
c
c  calculate solar correction factor for stomatal resistance
c
      gofs = 1. + (200./(gsun+0.1))**2
      rdc=100.*(1.+1000./(gsun+10.))
c
      tcell = tcellk - 273.0
      do 205 lu = 1,mlus
c
c      calculate temperature correction factor for stomatal resistance
c
        if (tcell .lt. tmin(lu)+.01 .or.
     &      tcell .gt. tmax(lu)-.01) then
          goft = 1.e4
        else
          tmxmn = tmax(lu) - tmin(lu)
          tmxo = tmax(lu) - topt(lu)
          tomn = topt(lu) - tmin(lu)
          goft = (tomn/(tcell - tmin(lu)))
     &           * (tmxo/(tmax(lu) - tcell))**(tmxo/tmxmn)
        endif
c
        do 200 l = 0,ngas
	  if ( l .eq. 0 ) then
	    ll=0
	  else
            ll = jind(l)
	  endif
          rcut(lu,l)=rcuts(lu,iseasn)/(hheff(ll)/heffso + freact(ll))
c
c       for now, stressed vegetation is set to 10 times stomatal
c       resistance otherwise calculated.  this may not be the 
c       most appropriate multiplier.
c
          if (lstrcl .eq. 1) then
            rstrf(lu,l)=dratio(ll)*rmin(lu,iseasn)*gofs*goft*10.
          else
            rstom(lu,l)=dratio(ll)*rmin(lu,iseasn)*gofs*goft
          endif
  200   continue
c
  205 continue
c
      do 280 l=0,ngas
	if ( l .eq. 0 ) then
	  ll=0
	else
          ll = jind(l)
	endif
        rmes(l) = 1./(hheff(ll)/3000. + 100.*freact(ll))
        do 270 lu=1,mlus
          if(flndcl(lu) .lt. 1.0e-3) then
            rsurf(lu,l) = 1.e10
            go to 270
          endif
          rcl(lu,l)=1./(hheff(ll)/(heffso*rcls(lu,iseasn))+freact(ll)/
     &              rclo(lu,iseasn))
          rgs(lu,l)=1./(hheff(ll)/(heffso*rgss(lu,iseasn))+freact(ll)/
     &              rgso(lu,iseasn))
c
c   check for presence of dew
c
          rtest = (100. - rhum) * (wsrf(lu)+0.6)
          ldew = .false.
          if (rtest .lt. 19.) ldew = .true.
c
c     note that the order of the following checks is important.
c     that is, water surface takes priority over rain wetted which
c     takes priority over dew wetted which takes priority over 
c     regular surface.
c
c        water surface
c
          if (lu .eq. 7) then
            rsurf(lu,l) = 1./(hheff(ll)*0.082*tcellk*4.8e-4*ustrw)
c
c        rain wetted surface
c
          else if (raincl .ge. 0.01) then
            if (l .eq. jso2) then
              if (lu .eq. 1) then
                rcutwet(lu,l) = 50.
              else
                rcutwet(lu,l) = 1./(1./5000. + 1./(3.*rcut(lu,l)))
              endif
            else if (l .eq. 0) then
              rcutwet(lu,l) = 1./(1./1000. + 1./(3.*rcut(lu,l)))
            else
              rcutwet(lu,l) = 1./(1./(3.*rcut(lu,l))+1e-7*hheff(ll) +
     &                    freact(ll)/rcutwet(lu,0))
            endif
            rsurf(lu,l)=1./(1./(3.*rstom(lu,l)+rmes(l))+1./rcutwet(lu,l)
     &                +1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
c
c        dew wetted surface
c
          else if (ldew) then
            if (l .eq. jso2) then
              if (lu .eq. 1) then
                rcutwet(lu,l) = 50.
              else
                rcutwet(lu,l) = 100.
              endif
            else if (l .eq. 0) then
              rcutwet(lu,l) = 1./(1./3000. + 1./(3.*rcut(lu,l)))
            else
              rcutwet(lu,l) = 1./(1./(3.*rcut(lu,l))+1e-7*hheff(ll) +
     &                    freact(ll)/rcutwet(lu,0))
            endif
            rsurf(lu,l)=1./(1./(3.*rstom(lu,l)+rmes(l))+1./rcutwet(lu,l)
     &                +1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
c
c        regular dry surface
c
          else
            if (lstrcl .le. 0) then
              rsurf(lu,l)=1./(1./(rstom(lu,l)+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            else if (lstrcl .eq. 1) then
              rsurf(lu,l)=1./(1./(rstrf(lu,l)+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            else if (lstrcl .ge. 2) then
              rsurf(lu,l)=1./(1./(10000.+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            endif
          endif
  270   continue
  280 continue
c
      do 305 n=1,mlus
        do 301 l=1,ngas
           rs(l,n) = rsurf(n,l)
           do nn=1,mxhg
             if (l .eq. khg0(nn)) rs(l,n) = 1.e20
           enddo
  301   continue
  305 continue
 315  continue
c
      return
      end
