c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine surlyr(time,ndate)
c
c-----ATDM v1.00
c
c     Performs the following tasks:
c     (1) estimates PBL parameters such as the monin-obukhov
c         length and ustar
c     (2) estimates dry deposition velocities at each grid cell
c         and for each species (gas and particle)
c         depvel - landuse-weighted deposition velocities (m/s)
c
c-----95/03/08:
c     -from UAM-V Revision 1.2  94/02/10  21:01:38  tcm
c     -modifications to streamline combined gas/particulate deposition
c      calculations
c-----95/10/17:
c     -revised calculation of cloud-reduced solar insolation
c-----95/11/03:
c     -revised rho in call to partdep to be in correct g/m3 units
c-----95/11/08:
c     -added code to skip cells covered by a fine grid
c-----96/09/13:dfg
c     -added call to micsurf instead of surfce when micro CB4 
c      mechanism is specified.
c     -added include 'ctlflg.cmd' for mechanism flag used in this routine.
c
c     Input arguments:
c               time:           model time       
c               ndate:          model date
c     Routines called:
c               solar     pblmod    aerody    bndry     surfce    partdep
c               jtocal
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'aerosl.cmd'
      include 'latlon.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      include 'deffin.cmd'
      include 'ctlflg.cmd'
      include 'advec.cmd'
c
      dimension ra(mlus), rs(mxgas,mlus), rb(mxgas,mlus)
      real zz0(mlus), solflx(10), cldadj(11), cosz(10), wsrf(mlus)
      dimension flndcl(mlus), vmeanp(mxspec,mlus)
      real lv
      logical lprt
c
      data zz0 /3., 0.25, 0.05, 1., 1., 1., 0.0001, 0.002, 
     &          0.15, 0.1, 0.1 /
      data solflx /915., 890., 840., 750., 630., 500., 345., 
     &          190., 60., 0./
      data cldadj /1., 0.91, 0.84, 0.79, 0.75, 0.72, 0.68, 
     &             0.62, 0.53, 0.41, 0.23/
      data cosz /1., 0.9962, 0.9659, 0.9063, 0.8192, 0.7071, 0.5736, 
     &           0.4226, 0.2588, 0.0872/
      data eps /0.622/, rd /287./, e0 /6.11/, lv /2.5e6/, rv /461./
      data iprt /79/, jprt /72/
      data iprt2 /55/, jprt2 /55/
c
cdiag
       write(*,*) 'just enter surlyr routine'
       call flush(6)
c-----Initialize some contants and arrays that are needed
c
      do 133 lu = 1,mlus
        ra(lu) = 0.
        do 131 l = 1,ngas
          rb(l,lu) = 0.0
          rs(l,lu) = 0.0
  131   continue
  133 continue
      do 136 l = 1,nospec
        do 135 j = 1,noy
          do 134 i = 1,nox
            depvel(i,j,l) = 0.
  134     continue
  135   continue
  136 continue
c
c-----Convert date to day,month and year
c
      if (ndate .lt. 100000) then
        call jtocal(ndate,icdate)
        ny = icdate/10000
        nm = icdate/100-ny*100
        nd = icdate-nm*100-ny*10000
      else
        ny = ndate/10000
        nm = ndate/100-ny*100
        nd = ndate-nm*100-ny*10000
      endif
      ny = ny+1900
c
c-----Calculate the surface deposition rate row by row
c
      do 100 j = 1,noy
c
c-----Check if out of modeling region
c
        if (istr(j).ge.nox) go to 100
        is = istr(j) + 1
        ie = iend(j) - 1
c
c-----Loop over each column element
c
        do 90 i = is,ie
          if (j.le.jstr(i)) goto 90
          if (j.ge.jend(i)) goto 90
c
c-----Skip cells covered by fine grid
c
          do 10 ifine = 1,numfin
            if (i.ge.ixfb(ifine) .and. i.le.ixfe(ifine) .and.
     &          j.ge.jyfb(ifine) .and. j.le.jyfe(ifine)) goto 90
 10      continue
          lprt = .false.
c         if (i.eq.iprt .and. j.eq.jprt) lprt = .true.
c         if (i.eq.iprt2 .and. j.eq.jprt2) lprt = .true.
c         if (lprt) then
c           write(*,*) 'Surlyr diagnostics for ', i, j
c           write(*,*) 'fland: ', (fland(i,j,lu),lu=1,11)
c         endif

c-----Calculate solar zenith angle and solar flux

          rllng = -ctlon(i,j) 
          rllt = ctlat(i,j)
          call solar(rllt,rllng,tz,ny,nm,nd,time,zcoef)
c         if (lprt) write(*,*)'lat,lon,tz,zcoef',
c    &                         rllt,rllng,tz,time,zcoef

          do 95 ncz = 2,10
            if (zcoef.ge.cosz(ncz).and.zcoef.lt.cosz(ncz-1)) then
              icz = ncz-1
              goto 96
            endif
   95     continue
          icz = 10

   96     continue 
          cldmax = 0.
          do 97 k = 1,nlayer
            cldmax = amax1(cldmax,cloud(i,j,k))
c           if (lprt) write(*,*) 'cloud ',k,cloud(i,j,k)
   97     continue
          ica = nint(10.*cldmax + 0.5)
          gsun = cldadj(ica)*solflx(icz)
c         if (lprt) then
c           write(*,*)'icz,cldmax,ica,cldadj,solflx,gsun'
c           write(*,*) icz,cldmax,ica,cldadj(ica),solflx(icz),gsun
c         endif
c
c-----Load environmental parameters; convert winds from km/min to m/s
c
          tcell = temper(i,j,1)
          pcell = press(i,j,1)
          tscell = tsurf(i,j)
          pscell = psfc(i,j)
          topcell = h(i,j,1)
          qcell = ch2o(i,j,1)
          if (lstagr) then
            ubar = (u(i,j,1) + u(i-1,j,1))/2.
            vbar = (v(i,j,1) + v(i,j-1,1))/2.
            wcell = 16.67*sqrt(ubar**2 + vbar**2)
            wcell = amax1(0.01,wcell)
          else
            wcell = 16.67*sqrt(u(i,j,1)**2 + v(i,j,1)**2)
            wcell = amax1(0.01,wcell)
          endif
c
          tv = tcell*(eps + qcell)/(eps*(1. + qcell))
          rhoair = 1.e5*pcell/(rd*tv)
          ev = qcell*pcell/(qcell + eps)
          es = e0*exp((lv/rv)*(1./273. - 1./tcell))
          rhum = 100.*ev/es
c         if (lprt) then
c           write(*,*)'t,p,ts,ps,top,q,w,rh,rho'
c           write(*,*)tcell,pcell,tscell,pscell,topcell,qcell,
c    &                wcell,rhum,rhoair
c         endif
c
c-----Loop over non-zero land use categories for each cell to get
c     gaseous ra & rb, and particle deposition velocity
c
          do 50 lu = 1,mlus
            flndcl(lu) = fland(i,j,lu)
            if (fland(i,j,lu).lt.1.0e-3) goto 50
            z00 = zz0(lu)
c           if (lprt) write(*,*)'doing LU # ',lu,fland(i,j,lu),z00
c
c-----Estimate the monin-obukhov length and ustar; save over-water ustar
c     for later use
c
cdiag    
c      call flush(6)
c
            call pblmod(wcell,tcell,tscell,pcell,pscell,topcell,
     &                  z00,elinv,ustar,wref)
            wsrf(lu) = wref
            if (lu.eq.7) then
              ustrw = ustar
              z00 = 1.6e-6/ustrw + 1.6e-3*ustrw**2
            endif
c           if (lprt) write(*,*)'elinv,ustar,wref,z00 ',
c    &                           elinv,ustar,wref,z00
c
c-----Estimate the aerodynamic resistance, and neutral 
c     ustar for later use
c
            call aerody(ustar,elinv,z00,ra(lu))
            ustrn = sqrt(wsrf(lu)/ra(lu))
c           if (lprt) write(*,*)'ra,ustrn',ra(lu),ustrn
c
c-----Estimate the laminar flow resistance for gases
c
cdiag
c      call flush(6)
c
            call bndry(ustrn,rb(1,lu))
cdiag
c      call flush(6)
c
c-----Calculate the deposition velocity of particles
c
            do 40 l = ngas+1,nospec
               if (l.eq.knh4n .or. l.eq.knh4s) goto 40
               isd = iszdist(l)
               rho = 1.e6*rhop(l)
               call partdep(rsize,rmass(1,isd),nbin,ustrn,
     &                      z00,rho,tcell,ra(lu),rhoair,
     &                      vmeanp(l,lu))
c              if (lprt) then
c                write(*,*)'rho,isd,vmeanp ',l,rho,isd,vmeanp(l,lu)
c              endif
   40       continue
   50     continue
c
c-----Estimate the canopy (surface) resistance for gases
c
c*** add call to micsurf when mechanism is specified. (dfg)
c
cdiag   
c       call flush(6)
c
        if (lmech) then
c
c      write (*,*) 'nm=', nm
c      write (*,*) 'gsun=', gsun
c      write (*,*) 'ustrw=', ustrw
c      write (*,*) 'wsrf=', wsrf
c      write (*,*) 'tcellk=', tcell
c      write (*,*) 'rhum=', rhum
c      write (*,*) 'lstrcl=', lstres(i,j)
c      write (*,*) 'lsnocl=', lsnow(i,j)
c      write (*,*) 'flndcl=',flndcl
c      write (*,*) 'raincl=', rain(i,j)
c      write (*,*) 'rs=', rs
c      write (*,*) 'pcell=', pcell
          call micsurf(nm,gsun,ustrw,wsrf,tcell,rhum,lstres(i,j),
     &                lsnow(i,j),flndcl,rain(i,j),rs,pcell)
        else
          call surfce(nm,gsun,ustrw,wsrf,tcell,rhum,lstres(i,j),
     &                lsnow(i,j),flndcl,rain(i,j),rs)
        endif
c         if (lprt) write(*,*)'lstrs,lsnow,rain ',
c    &                         lstres(i,j),lsnow(i,j),rain(i,j)
c
cdiag
c       call flush(6)
c
c-----Calculate the deposition velocity (m/s)
c     running sum over landuse becomes landuse-weighted deposition velocity
c     for species k
c
          do 85 k = 1,ngas
             do 55 lu = 1,mlus
                if (fland(i,j,lu).lt.1.e-3) goto 55
                vdep = 1.0/(ra(lu) + rb(k,lu) + rs(k,lu))
                depvel(i,j,k) = depvel(i,j,k) - fland(i,j,lu)*vdep
c               if (lprt) write(*,*)'i,j,k,lu,ra,rb,rs,vdep ',
c    &                               i,j,k,lu,ra(lu),rb(k,lu),rs(k,lu),vdep
c                write(*,*)'for gas- k,lu,ra,rb,rs,vdep,depvel ',
c     &                               i,j,k,lu,ra(lu),rb(k,lu),rs(k,lu),vdep,depvel(i,j,k)
  55         continue
c            if (lprt) write(*,*)'depvel ',k,depvel(i,j,k)
  85      continue
          do 86 k = ngas+1,nospec
             if (k.eq.knh4n .or. k.eq.knh4s) goto 86
             do 56 lu = 1,mlus
                if (fland(i,j,lu).lt.1.e-3) goto 56
                vdep = vmeanp(k,lu)
                depvel(i,j,k) = depvel(i,j,k) - fland(i,j,lu)*vdep
  56         continue
c            if (lprt) write(*,*)'depvel ',k,depvel(i,j,k)
  86      continue
          if (knh4n.ne.0) depvel(i,j,knh4n) = depvel(i,j,kpno3)
c
   90   continue
  100 continue
c
      return
      end
