c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine toxchm(dt,tox,temp)
c     subroutine toxchm(dt,tox,temp,oh)
c
c-----ATDM v1.00
c
c     Calculates chemical decay/production of toxic species 
c     Exponential decay over time dt is specified for NAPH, DIOXIN,
c     and ATRAZ.  POM1 and POM2 are produced as a result of other reactions
c     so more complicated expressions for these are used.
c
c-----95/07/18:
c     -from UAM6.40tox, 3/16/95
c     -Modified for ATDM toxic species
c-----95/07/21:
c     -Revised to check for specific reactions
c-----95/08/07:
c     -revisions to ensure consistent use of temperature dependence in RK's
c-----95/09/28:
c     -changed decay rate for DIOXIN from 1.5e4 to 3.0e3
c-----96/09/17:dfg
c     -removed argument oh because it include in common chrad in comchem.cmd.
c
c     Input arguments:
c               dt              Model timestep (min)
c               tox             Concentration of 5 Gas-phase Toxic species
c                               (ppm): NAPH POM1 POM2 DIOXIN ATRAZ
c               temp            Ambient temperature (K)
c               oh              Concentration of OH radical (ppm)
c     Output arguments:
c               tox             Concentration of 5 Gas-phase Toxic species
c     Called by:
c               rchem     rchemf
c
      include 'parchm.cmd'
      include 'comchem.cmd'
c
      dimension tox(5)
c
      data rknaph0 /6.0e+4/, tdepnaph /-440./
      data rkdiox0 /3.0e+3/
      data rkatrz0 /2.0e+5/
      data rkpom10 /7.0e+4/, tdeppom1 /-440./
c
      tfac = 1./298. - 1./temp
c
c-----NAPH/POM1/POM2 Chemistry
c
c      write (*,*) 'decay of naph', knaph,kpom1,kpom2,kdiox,katraz
c  decay of NAPH
c
      if (knaph.ne.0 .and. kpom1.ne.0 .and. kpom2.ne.0) then
        rknaph = rknaph0*exp(tdepnaph*tfac)
        rnaph = 0.5*rknaph*oh
        enaph = exp(-rnaph*dt)
        cnaph = tox(1)*enaph
c        write (*,*) rknaph,rknaph0,tfac,tdepnaph
c
c  decay and production of POM1
c
        rkpom1 = rkpom10*exp(tdeppom1*tfac)
        rpom1 = 0.5*rkpom1*oh
c        write (*,*) rkpom1,rkpom10,tdeppom1
c        write (*,*) 'decay of pom1',rnaph,oh,rpom1
        rprni = 1./(rpom1-rnaph)
        rtwo  = 0.02*rknaph*oh*tox(1)
        epom1 = exp(-rpom1*dt)
        cpom1 = tox(2)*epom1 + rtwo*rprni*(enaph-epom1)
c
c  production of POM2
c
        rpom2 = 0.1*rkpom1*oh
c        write (*,*) 'decay of pom2',rpom2
        cpom2 = tox(3) + (rpom2/rpom1)*(tox(2)*(1.-epom1)
     &       + (rtwo/rnaph)*(1. + rnaph*rprni*epom1 -
     &       rpom1*rprni*enaph))
c
        tox(1) = cnaph
        tox(2) = cpom1
        tox(3) = cpom2
      endif
c
c      write (*,*) 'decay of dioxin'
c-----Decay of DIOXIN
c
      if (kdiox.ne.0) then
        rdiox = rkdiox0*oh
        ediox = exp(-rdiox*dt)
        cdiox = tox(4)*ediox
        tox(4) = cdiox
      endif
c
c      write (*,*) 'decay of atraz'
c-----Decay of ATRAZ
c
      if (katraz.ne.0) then
        ratrz = rkatrz0*oh
        eatrz = exp(-ratrz*dt)
        catrz = tox(5)*eatrz
        tox(5) = catrz
      endif
c
      return
      end
