c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine vertwd (time,ndate,idebug)
c
c-----ATDM v1.00
c
c     Calculates vertical sigma velocity on the coarse grid
c     (d(sigma)/dt = sigdot)
c
c-----95/06/07:
c     C Emery SAI
c-----95/06/13:
c     -Revised layer index for div from k to k+1 in calculation of sigdot
c     -pppt now based on rate of change in P* as a result of input surface P,
c      rather than calculated based on column-integrated divergence.  This
c      should improve mass consistency
c-----95/06/15:
c     -added conversion of km to m for deltax, deltay and tphioa
c-----95/10/04:
c     -fixed erroneous calculation of div by adding missing dsigma factor
c-----95/11/08:
c     -added code to skip cells covered by a fine grid
c
c-----96/10/31:
c     -pppt calculated from column-integrated divergence... This is more
c      more appropriate for this routine due to time-scale considerations
c
c     Input arguments:
c               time:           model time
c               ndate:          model date
c               idebug:         diagnostic debug flag
c     Files written:
c               6:              standard output file unit (*)    
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'com3.cmd'
      include 'advec.cmd'
      include 'deffin.cmd'
      dimension div(mxz)
c
c  bit 13 (4096's) of idebug turns on debugs in update and vertwd
c
      idiag = and(idebug,4096)
c
      if (idiag .gt. 0) then
        write(*,*) ' calculate divergence'
      endif
c
c     loop through grid
c
      do 100 j = 1,noy
c
c     check for outside of modeling region
c
        if (istr(j).ge.nox) go to 100
        i1 = istr(j)+1
        i2 = iend(j)-1
        do 200 i = i1,i2
          if (j.le.jstr(i)) go to 200
          if (j.ge.jend(i)) go to 200
c
c-----Skip cells covered by fine grid
c
          do 10 ifine = 1,numfin
            if (i.ge.ixfb(ifine) .and. i.le.ixfe(ifine) .and.
     &          j.ge.jyfb(ifine) .and. j.le.jyfe(ifine)) goto 200
  10      continue
c
c     local time rate of change of P* is based on change in input surface
c     pressure; convert sfc pressure from mb to Pa, and datadt from min to s
c
c          pppt = 100.*(psnew(i,j) - psold(i,j))/(60.*datadt)
c
c     loop through all layers to calculate divergence; convert winds from
c     km/min to m/s, and pstar from mb to Pa
c
          sum = 0.
          do 300 k=nlayer,1,-1
            dsigma = sigma(k) - sigma(k+1)
            if (lstagr) then
              uip = 16.67*u(i,j,k)
              uim = 16.67*u(i-1,j,k)
              vjp = 16.67*v(i,j,k)
              vjm = 16.67*v(i,j-1,k)
              vmid = 16.67*(v(i,j,k) + v(i,j-1,k))/2.
            else
              uip = 16.67*(u(i,j,k) + u(i+1,j,k))/2.
              uim = 16.67*(u(i,j,k) + u(i-1,j,k))/2.
              vjp = 16.67*(v(i,j,k) + v(i,j+1,k))/2.
              vjm = 16.67*(v(i,j,k) + v(i,j-1,k))/2.
              vmid = 16.67*v(i,j,k)
            endif
            pip = 100.*pstar(i,j)
            pim = pip
            pjp = pip
            pjm = pip
            if (uip.lt.0.) pip = 100.*pstar(i+1,j)
            if (uim.ge.0.) pim = 100.*pstar(i-1,j)
            if (vjp.lt.0.) pjp = 100.*pstar(i,j+1)
            if (vjm.ge.0.) pjm = 100.*pstar(i,j-1)
            divu = (pip*uip - pim*uim)/(1000.*deltax(j))
            divv = (pjp*vjp - pjm*vjm)/(1000.*deltay)
            divlat = -100.*pstar(i,j)*vmid*tphioa(j)/1000.
            div(k) = (divu + divv + divlat)*dsigma
            sum = sum - div(k)
  300     continue
          pppt = sum
c
c     calculate vertical sigma velocity
c
          sum = 0.
          sigdot(i,j,nlayer) = 0.
          psij = 100.*pstar(i,j)
          do 301 k = nlayer-1,1,-1
            sum = sum - div(k+1) 
            sigdot(i,j,k) = sum/psij - sigma(k+1)*pppt/psij
  301     continue
  200   continue
  100 continue
c
      return
      end
