c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine xmscal(mdx,mdy,mdz,mdspc,nxg,nyg,nzg,
     &                  delx,dely,cnca,ifine)
c
c-----ATDM v1.00
c
c     Quick mass budget calculation for portion of coarse grid
c     covered by a specific fine grid.
c
c-----95/06/07:
c     -from UAM-V Revision 1.2  94/02/10  21:03:02  tcm
c     -conversion now from ppm to moles
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c     -removed hta from argument list
c     -added data statements for grav and amolwt
c
c     Input arguments:
c               mdx:            max number of grid cells in x-direction     
c               mdy:            max number of grid cells in y-direction     
c               mdz:            max number of layers 
c               mdspc:          max number of species
c               nxg:            number of grid cells in x-direction
c               nyg:            number of grid cells in y-direction
c               nzg:            number of layers 
c               delx:           cell size in x-direction (km)
c               dely:           cell size in y-direction (km)
c               cnca:           array of concentrations (ppm)
c               ifine:          fine grid index
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'balanc.cmd'
c
      dimension cnca(mdx,mdy,mdspc,mdz)
      dimension delx(nyg)
      real mfac
      data grav /9.8/, amolwt /28.8/
c
c  get start and end cells in coarse grid of this fine grid
c
      ix1 = ixfb(ifine)
      ix2 = ixfe(ifine)
      jy1 = jyfb(ifine)
      jy2 = jyfe(ifine)
c
      do 5530 l=1,nospec
        xtmas(l,ifine) = 0.
        do 5533 k=1,nzg
          dsigma = sigma(k) - sigma(k+1)
          do 5534 j=jy1,jy2
            do 5535 i=ix1,ix2
              vfac = 1.e6*delx(j)*dely*dsigma
              mfac = pstar(i,j)/(10.*grav*amolwt)
              xtmas(l,ifine) = xtmas(l,ifine) +
     &                         cnca(i,j,l,k)*vfac*mfac
 5535       continue
 5534     continue
 5533   continue
 5530 continue
      return
      end
